/*******************************************************************************
 * (c) Copyright 2011 Actel Corporation.  All rights reserved.
 * 
 * Public function declarations, register and bit level definitions for 
 * Marvell's M88E1111 PHY interface.
 * This file contains bit definitions of only M88E1111 specific registers. 
 * For bit definitions of other (standard MII) registers, refer phy.h.
 * 
 * 
 * SVN $Revision: 4791 $
 * SVN $Date: 2012-12-06 10:20:29 +0000 (Thu, 06 Dec 2012) $
 */
#ifndef M88E1111_PHY_H_
#define M88E1111_PHY_H_

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------
 * M88E1111 PHY address
 */
#define M88E1111_PHY_ADDR                   0x07

/*------------------------------------------------------------------------------
 * M88E1111 PHY specific register offsets and bit definitions
 */
#define M88E1111_CR         0x00
 
#define M88E1111_PHY_SCR		            0x10
#define M88E1111_PHY_SCR_AUTO_CROSS	    0x0060

#define M88E1111_PHY_LED_CONTROL	        0x18
#define M88E1111_PHY_LED_DIRECT	        0x4100
#define M88E1111_PHY_LED_COMBINE	        0x411c

#define M88E1111_PHY_EXT_CR		        0x14
#define M88E1111_RX_DELAY		            0x80
#define M88E1111_TX_DELAY		            0x2

#define M88E1111_EXT_ADDR_PAGE_CR       0x16
#define PAGE_0                          0x00
#define PAGE_1                          0x01

#define M88E1111_PHY_EXT_SR		        0x1b

#define M88E1111_HWCFG_MODE_MASK		    0xf
#define M88E1111_HWCFG_MODE_COPPER_RGMII	0xb
#define M88E1111_HWCFG_MODE_FIBER_RGMII	0x3
#define M88E1111_HWCFG_MODE_SGMII_NO_CLK	0x4
#define M88E1111_HWCFG_MODE_COPPER_RTBI	0x9
#define M88E1111_HWCFG_FIBER_COPPER_AUTO	0x8000
#define M88E1111_HWCFG_FIBER_COPPER_RES	0x2000

#define M88E1111_COPPER		            0
#define M88E1111_FIBER			            1

#define M88E1111_PHY_STATUS		            0x11
#define M88E1111_PHY_STATUS_1000	        0x8000
#define M88E1111_PHY_STATUS_100	            0x4000
#define M88E1111_PHY_STATUS_SPD_MASK	    0xc000
#define M88E1111_PHY_STATUS_FULLDUPLEX	    0x2000
#define M88E1111_PHY_STATUS_RESOLVED	    0x0800
#define M88E1111_PHY_STATUS_LINK	        0x0400

/**************************************************************************/
/* Public Function declarations                                           */
/**************************************************************************/

/***************************************************************************//**
  The function m88e1111_init()is used to initialize Marvell's M88E1111 PHY 
  interface. This function is called from MSS_MAC_init() function when
  M88E1111 PHY is the selected as PHY interface. PHY interface callback 
  mechanism defined in PHY_devices of phy.c is used to make this funciton call. 
  This function resets the M88E1111 interface, disables loopback, enables 
  auto-negotiation and restarts the same when auto-negotiation is enabled in 
  the MAC configuration record. 
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer.
 
  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"

    PHY_device[cfg->phy].init( cfg );
 */
return_t 
m88e1111_init
(
    MAC_cfg_t * cfg
);

/***************************************************************************//**
  The function m88e1111_config_aneg() is used to configure and perform auto
  negotiation for M88E1111 PHY interface. This function is called from MSS_MAC_init()
  using PHY_device callback mechanism when the PHY interface is chosen as 
  M88E1111 by the user. This functions advertises the capabilities configured by 
  the user after complying with the M88E1111 capabilities. Then it restarts auto
  negotiation and waits till it is completed. This function internally polls the 
  "auto negotiation complete" flag of M88E1111 status register and returns once
  the flag indicates that auto negotiation is completed or timed out.
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer.

  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"

    PHY_device[cfg->phy].config_aneg( cfg );
  @endcode
 */
return_t 
m88e1111_config_aneg
(
    MAC_cfg_t * cfg
);

/***************************************************************************//**
  The function m88e1111_read_status() is used to read the status of the M88E1111
  PHY once initiated auto-negoation is completed. This function uses the cfg
  parameter as out param. This function reads the link partner ability register
  of the PHY to find the negotiated abilities. Accordingly some of the elements
  of configuration record will be modified, which will be used in initialization 
  of Ethernet MAC. 
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer. Some elements of this record will be modified
    after successful execution of this function. 

  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"
            
    PHY_device[cfg->phy].read_status( cfg );
  @endcode
 */
return_t 
m88e1111_read_status
(
    MAC_cfg_t * cfg
);


#ifdef __cplusplus
}
#endif

#endif /* M88E1111_PHY_H_ */


