/*******************************************************************************
 * (c) Copyright 2011 Actel Corporation.  All rights reserved.
 *
 * G4Main Microcontroller Subsystem 10/100/1000 Mbps Ethernet MAC bare metal
 * software driver public APIs.
 *
 * SVN $Revision: 4791 $
 * SVN $Date: 2012-12-06 10:20:29 +0000 (Thu, 06 Dec 2012) $
 */
/*=========================================================================*//**
  @mainpage G4Main MSS Ethernet MAC Bare Metal Driver.

  @section intro_sec Introduction
    The G4Main microcontroller subsystem (MSS) includes one 10/100/1000 Mbps 
    Ethernet MAC hardware peripheral. This MAC supports MII/RMII/GMII/RGMII/
    SGMII interfaces to the physical layer devices (PHY). 

    The G4Main MSS Ethernet MAC bare metal software driver is made to work with
    this Ethernet MAC hardware. This driver provides a set of interface functions
    that can be used in the applications where no operating system is available. 
    These drivers can be adapted for use as part of an operating system, but the 
    implementation of the adaptation layer between this driver and the operating 
    system's driver model is outside the scope of this driver.

  @section hw_dependencies Hardware Flow Dependencies
    The configuration of all features of the MAC is covered by this driver
    with the exception of the number of transmitter and receiver buffers required
    to cope up with the network traffic. This configuration can be done at the 
    time of generating this driver from Firmware Catalog. While generating this 
    driver, a GUI provides options to chose number of transmitter and receiver 
    buffers. This configuration will be used to create definitions in 
    mss_ethernet_mac_user_config.h file. These definitions will be used in the 
    driver to create corresponding transmitter and receiver descriptor rign buffers.

    Apart from this, if the designer decides to use either MII/GMII/SGMII 
    PHY interfaces then the correponding IOMUXes in the MSS should be configured
    accordingly. These interfaces does not have pins assigned at MSS level and
    always inted to route through fabric of G4Main. The other two interfaces RMII 
    and RGMII have assigned pins in the MSS for which the IOMUXes will be configured
    automatically when these interfaces are chosen.

  @section theory_op Theory of Operation
    The G4Main Ethernet MAC driver functions are logically grouped into four
    groups:
    - Initialization and configuration.
    - Transmit and receive operations.
    - Interrupt configuration.
    - Reading status and statistics.

    Initialization and Configuration
    The G4Main Ethernet MAC driver is initialized by calling MSS_MAC_init()
    function with a parameter which is reference to the MAC_cfg_t type 
    configuration record. This configuration record contains configuration 
    information like speed, duplex mode, autonegotiation and other ethernet 
    hardware configuration related options. The MSS_MAC_init() function takes 
    these parameters and configures the MAC hardware and driver accordingly. It 
    is always advised to use MSS_MAC_cfg_struct_def_init() function to initialize 
    the MAC_cfg_t type configuration record before passing to the initialization. 
    This makes sure that the MAC configured with default values and makes the 
    driver operational. This initialization process assigns a default MAC address
    (Station address) to the system. You can change this by calling 
    MSS_MAC_set_mac_addr() function. This function accepts a parameter which is 
    uint8_t pointer to the MAC address array and assigns the address to the station.
    Below is the list of functions which are needed for initialization and 
    configuration process:
    - MSS_MAC_cfg_struct_def_init()
    - MSS_MAC_init()
    - MSS_MAC_set_mac_addr()
    
    Packet Management
    It is required to provide an effective packet management mechanism 
    between the application and the MAC driver to support both non-blocking 
    and blocking transactions. This is achieved by providing a type MAC_pkt_t 
    for creating packet container buffer. You must create transmit and receive 
    packet containers in the application memory space each of size 
    MSS_MAC_TX_RING_SIZE and MSS_MAC_RX_RING_SIZE respectively. For example:
        MAC_pkt_t tx_pkts[MSS_MAC_TX_RING_SIZE];
        MAC_pkt_t rx_pkts[MSS_MAC_TX_RING_SIZE];
    It is always good to create these packet containers statically. It is 
    required to call MSS_MAC_init_pkt_pool() function with appropriate 
    parameters to initialize these packet container pools. You can get the 
    next free and available packet by calling the function MSS_MAC_alloc_pkt() 
    with appropriate parameters.
    These packet containers contain three elements lock, buffer and size. 
    To the application, the lock element has the possible states of LOCK, 
    UNLOCK and EMPTY for receive container and LOCK, UNLOCK and FULL states 
    for the transmit container. It is necessary to keep this lock parameter 
    in appropriate state before using it for transmission and reception. For 
    this, the driver provides API functions. 
    The buffer parameter is a pointer to unsigned char buffer of size 
    MSS_MAC_MAX_TX_BUF_SIZE in case of transmit buffer and MSS_MAC_MAX_RX_BUF_SIZE 
    in case of receive buffer. You need to create these unsigned char buffers 
    either statically or dynamically and assign to the buffer parameter of the 
    packet container that is used for transmission or reception. The driver 
    provides API functions to transfer transmit and receive packets into these 
    buffers and change the state of lock parameter accordingly such that the 
    packet container can be used another transmission or reception. 
    Below are set of API functions provided by the driver for these operations:
    - MSS_MAC_init_pkt_pool()
    - MSS_MAC_alloc_pkt()
    - MSS_MAC_transfer_rx_pkt()
    - MSS_MAC_prepare_tx_pkt()

    PHY Management
    The initialization of the MAC hardware and driver requires interaction with 
    the physical layer devices (PHYs) that connect the MAC to the physical 
    medium. As these PHYs are usually vendor specific, they vary in features 
    and configuration across different hardware boards. But for the fundamental 
    configuration and auto-negotiation process, the 802.3 standard provides the 
    MDIO protocol to communicate with the PHYs. The G4Main Ethernet MAC driver 
    makes use of this MDIO protocol to provide a generic PHY management scheme 
    that avoids code level changes in the driver when external PHY device is changed. 
    For this a generic PHY device type PHY_dev_t is provided. This type contains 
    necessary attributes of the PHY device which are mandatory for auto-negotiation 
    process. Refer to PHY_dev_t description for details. The MAC driver uses these 
    PHY device attributes to perform auto-negotiation and auto-detection while 
    initializing. 
    To support a new PHY device, you can write interface functions complies 
    the PHY_dev_t attributes and add in the PHY_dev_t PHY_device record. The PHY 
    that is newly supported should be added in the PHY_type_t enumeration and 
    should be used in the configuration used for initialization of the driver. 
    To support a new PHy device, the MAC driver provides generic routines to 
    interact using standard MDIO protocol.  You can use these routines in 
    developing the custom PHY driver. The set of APIs provided are:
    - MSS_MAC_read_phy_reg()
    - MSS_MAC_write_phy_reg()
    - MSS_MAC_read_phy_indicators()
    These routines make use of the MDIO interface feature that is supported in 
    the G4Main Ethernet MAC hardware.
    The phy.c source file contains a generic implementation of MDIO protocol 
    that can be used to configure most of the PHY devices. 
    
    Transmit and Receive operations
    The G4Main Ethernet MAC driver provides a set of APIs which are mainly 
    targeted at non-blocking execution for transmit and receive operations. 
    But by using a combination of these APIs, you can create blocking APIs 
    which work based on polling methodology. Internally, the driver relies 
    on DMA and related interrupts for execution of these APIs. Below is the 
    set of APIs for transmit and receive operations.
    - MSS_MAC_is_pkt_received()
    - MSS_MAC_is_pkt_transmitted()
    - MSS_MAC_receive_pkt()
    - MSS_MAC_send_pkt()
    - MSS_MAC_strat_tx()
    - MSS_MAC_stop_tx()
    - MSS_MAC_start_rx()
    - MSS_MAC_stop_rx()

    Interrupt Configuration
    It is necessary to enable MAC interrupts to effectively transmit and 
    receive packets through the MAC, to utilize the automatic DMA descriptor 
    and FIFO management mechanism of MAC hardware. For this purpose the 
    Ethernet MAC driver provides below set of APIs.
    - MSS_MAC_enable_irq()
    - MSS_MAC_disable_irq()
    - MSS_MAC_assign_irq_handler()

    The MAC driver assigns default handlers for transmit and receive interrupts 
    which will work to transmit and receive packets. These default handlers work 
    for non-blocking transactions as is. But in case if you want to create 
    custom interrupt handlers, you have use MSS_MAC_assign_irq_handler() function. 
    This needs thorough understanding of packet management in the driver. There is 
    no need to alter and rewrite transmit and receive functions.

    Reading Status and Statistics
    It is necessary to read status and statistics during transmit and receive 
    transactions of MAC operations. The MAC hardware provides transmit and receive 
    status in bit encoded format and the statistics as counters for transmission 
    and receptions separately. This driver provides below set of APIs for reading 
    status and statistics.
    - MSS_MAC_read_tx_status()
    - MSS_MAC_read_rx_status()
    - MSS_MAC_read_interface_status()
    - MSS_MAC_read_stat()
    - MSS_MAC_read_tx_pkt_count()
    - MSS_MAC_read_rx_pkt_count()

 *//*=========================================================================*/
#ifndef MSS_ETHERNET_MAC_H_
#define MSS_ETHERNET_MAC_H_

#include "mss_ethernet_mac_types.h"

#ifdef __cplusplus
extern "C" {
#endif

/*******************************************************************************
 * State definitions
 */
#define DISABLE		                        ( (uint8_t)0 )
#define ENABLE                              ( (uint8_t)!DISABLE )

/*******************************************************************************
 * API Function return value definitions
 */
#define SUCCESS   ENABLE
#define FAILED    DISABLE

/*******************************************************************************
 * Default MAC address 
 */
#define MSS_MAC_DEFAULT_MAC_ADDRESS         0xC0,0xB1,0x3C,0x88,0x88,0x88

/*******************************************************************************
 * Broadcast MAC address 
 */
#define MSS_MAC_BROADCAST_MAC_ADDRESS 	    0xFF,0xFF,0xFF,0xFF,0xFF,0xFF

/*******************************************************************************
 * Maximum MAC frame size (packet size)
 */
#define MSS_MAC_MAX_PACKET_SIZE             1536u

/*******************************************************************************
 * Per Packet Overrides masks in descriptor packet size parameters
 */
#define MSS_MAC_FIFO_TX_CTRL_FRAME			( (uint32_t)0x00100000 )
#define MSS_MAC_FIFO_TX_NO_CTRL_FRAME       ( ~MSS_MAC_FIFO_TX_CTRL_FRAME )
#define MSS_MAC_FIFO_TX_PERPKT_PAD_FCS		( (uint32_t)0x00040000 )
#define MSS_MAC_FIFO_TX_PERPKT_NO_PAD_FCS	( ~MSS_MAC_FIFO_TX_PERPKT_PAD_FCS )
#define MSS_MAC_FIFO_TX_PERPKT_FCS			( (uint32_t)0x00020000 )
#define MSS_MAC_FIFO_TX_PERPKT_NO_FCS       ( ~MSS_MAC_FIFO_TX_PERPKT_FCS )
#define MSS_MAC_FIFO_TX_PERPKT_ENABLE		( (uint32_t)0x00010000 )
#define MSS_MAC_FIFO_TX_PERPKT_DISABLE      ( ~MSS_MAC_FIFO_TX_PERPKT_ENABLE )

/*******************************************************************************
 * Transmit and receive packet buffer sizes
 */
#define MSS_MAC_MAX_TX_BUF_SIZE             MSS_MAC_MAX_PACKET_SIZE
#define MSS_MAC_MAX_RX_BUF_SIZE             MSS_MAC_MAX_PACKET_SIZE

/*******************************************************************************
 * Defines for configuration parameters
 */
/* Auto-negotiation enable / disable */
#define MSS_MAC_AUTO_NEGOTIATION_ENABLE     		ENABLE
#define MSS_MAC_AUTO_NEGOTIATION_DISABLE    		DISABLE

/* FIFO error detection & correction enable / disable */
#define MSS_MAC_ERR_DET_CORR_ENABLE         		ENABLE
#define MSS_MAC_ERR_DET_CORR_DISABLE          		DISABLE

/* Huge frame support enable / disable */
#define MSS_MAC_HUGE_FRAME_ENABLE           		ENABLE
#define MSS_MAC_HUGE_FRAME_DISABLE          		DISABLE

/* Length field checking enable / disable */
#define MSS_MAC_LENGTH_FILED_CHECK_ENABLE   		ENABLE
#define MSS_MAC_LENGTH_FILED_CHECK_DISABLE  		DISABLE

/* Padding and CRC enable / disable */
#define MSS_MAC_PAD_N_CRC_ENABLE            		ENABLE
#define MSS_MAC_PAD_N_CRC_DISABLE           		DISABLE

/* Append CRC enable / disable */
#define MSS_MAC_CRC_ENABLE                  		ENABLE
#define MSS_MAC_CRC_DISABLE                 		DISABLE

/* Fullduplex mode enable / disable */
#define MSS_MAC_FULLDUPLEX_ENABLE           		ENABLE
#define MSS_MAC_FULLDUPLEX_DISABLE          		DISABLE

/* Loopback mode enable / disable */
#define MSS_MAC_LOOPBACK_ENABLE            			ENABLE
#define MSS_MAC_LOOPBACK_DISABLE            		DISABLE

/* Receiver flow control enable / disable */
#define MSS_MAC_RX_FLOW_CTRL_ENABLE         		ENABLE
#define MSS_MAC_RX_FLOW_CTRL_DISABLE        		DISABLE

/* Transmission flow control enable / disable */
#define MSS_MAC_TX_FLOW_CTRL_ENABLE         		ENABLE
#define MSS_MAC_TX_FLOW_CTRL_DISABLE        		DISABLE

/* Excessive defer enable / disable */
#define MSS_MAC_EXSS_DEFER_ENABLE           		ENABLE
#define MSS_MAC_EXSS_DEFER_DISABLE          		DISABLE

/* No-backoff enable / disable */
#define MSS_MAC_NO_BACKOFF_ENABLE           		ENABLE
#define MSS_MAC_NO_BACKOFF_DISABLE          		DISABLE

/* Backpressure no-backoff enable / disable */
#define MSS_MAC_BACKPRESS_NO_BACKOFF_ENABLE         ENABLE
#define MSS_MAC_BACKPRESS_NO_BACKOFF_DISABLE        DISABLE

/* Alternative binary exponential backoff enable / disable */
#define MSS_MAC_ABEB_ENABLE                         ENABLE
#define MSS_MAC_ABEB_DISABLE                        DISABLE

/* Supress preamble enable / disable */
#define MSS_MAC_SUPPRESS_PREAMBLE_ENABLE            ENABLE
#define MSS_MAC_SUPPRESS_PREAMBLE_DISABLE           DISABLE

/* Auto-scan PHYs enable / disable */
#define MSS_MAC_PHY_AUTOSCAN_ENABLE                 ENABLE
#define MSS_MAC_PHY_AUTOSCAN_DISABLE                DISABLE

/* Preamble length default value and maximum value */
#define MSS_MAC_PREAMLEN_DEFVAL                     ( (uint8_t)(0x07) )
#define MSS_MAC_PREAMLEN_MAXVAL                     ( (uint8_t)(0x0F) )

/* Byte / Nibble mode  */
#define MSS_MAC_NIBBLE_MODE                         ( (uint8_t)(0x00) )
#define MSS_MAC_BYTE_MODE                           ( (uint8_t)(0x01) )

/* IPG/IFG values */
#define MSS_MAC_MINIFG_MAXVAL                       ( (uint8_t)(0xFF) )
#define MSS_MAC_MINIFG_DEFVAL                       ( (uint8_t)(0x50) )
#define MSS_MAC_BTBIFG_MAXVAL                       ( (uint8_t)(0x7F) )
#define MSS_MAC_BTBIFG_DEFVAL                       ( (uint8_t)(0x60) )
#define MSS_MAC_NONBTBIFG_DEFVAL                    ( (uint16_t)((0x4000) | 0x0060) )
#define MSS_MAC_NONBTBIFG_MAXVAL                    ( (uint16_t)(0x3FFF) )

/* Number of maximum retransmission tries */
#define MSS_MAC_MAXRETX_MAXVAL                      ( (uint8_t)(0x1F) )
#define MSS_MAC_MAXRETX_DEFVAL                      ( (uint8_t)(0x0F) )
#define MSS_MAC_ABEBTRUNC_MAXVAL                    ( (uint8_t)(0x0F) )
#define MSS_MAC_ABEBTRUNC_DEFVAL                    ( (uint8_t)(0x0A) )

/* PHY clock divider values */
#define MSS_MAC_DEF_PHY_CLK                         ( (uint8_t)(0x07) )
#define MSS_MAC_BY4_PHY_CLK                         ( (uint8_t)(0x01) )
#define MSS_MAC_BY6_PHY_CLK                         ( (uint8_t)(0x02) )
#define MSS_MAC_BY8_PHY_CLK                         ( (uint8_t)(0x03) )
#define MSS_MAC_BY10_PHY_CLK                        ( (uint8_t)(0x04) )
#define MSS_MAC_BY14_PHY_CLK                        ( (uint8_t)(0x05) )
#define MSS_MAC_BY20_PHY_CLK                        ( (uint8_t)(0x06) )
#define MSS_MAC_BY28_PHY_CLK                        ( (uint8_t)(0x07) )

/* Default & Maximum PHY addresses */
#define MSS_MAC_DEFAULT_PHY                         ( DP83848 )
#define MSS_MAC_PHYADDR_MAXVAL                      ( (uint8_t)(0x1F) )
#define MSS_MAC_PHYREGADDR_MAXVAL                   ( (uint8_t)(0x1F) )
#define MSS_MAC_PHY_ADDR_DEFVAL                     ( (uint8_t)(0x00) )
#define MSS_MAC_PHYREGADDR_DEFVAL                   ( (uint8_t)(0x00) )

/* Maximum frame length default & maximum values */
#define MSS_MAC_MAXFRAMELEN_DEFVAL                  ( (uint16_t)(0x0600) )
#define MSS_MAC_MAXFRAMELEN_MAXVAL                  ( (uint16_t)(0xFFFF) )

/* Slottime (Collision window) default & maximum values */
#define MSS_MAC_SLOTTIME_DEFVAL                     ( (uint16_t)(0x0037) )
#define MSS_MAC_SLOTTIME_MAXVAL                     ( (uint16_t)(0x03FF) )

/* Frame drop mask default value */                                         
#define MSS_MAC_FRAME_DROP_MASK_DEFVAL      (        (uint32_t)(0x00000000) )


/**************************************************************************/
/* Global variables                                                       */
/**************************************************************************/

/*
 * This is global instance for Ethernet MAC. This works as a container for
 * all MAC related attributes like descriptor rings, h/w registers and 
 * current station address assigned to the MAC.
 */
extern MAC_instance_t g_mac;


/**************************************************************************/
/* Public Function declarations                                           */
/**************************************************************************/

/***************************************************************************//**
  The function MSS_MAC_cfg_struct_def_init() initializes the configuration 
  record of type MAC_cfg_t, created in application memory space to default
  values of configuration parameters. The default parameters are those which
  are enable the MAC interface to function with 100Mbps, half-duplex and RGMII 
  based interface. 
 
  @param cfg
  This is pointer to the configuration record in application memory space. You
  should create and maintain a configuration record of MAC_cfg_t that holds
  the MAC configuration required for the application.
 
  @return
  This function returns 1 on success and 0 on failure.

  Example:
  @code
  MAC_cfg_t mycfg;

  if(MSS_MAC_cfg_struct_def_init(&mycfg))
  {
    // configuration successful.
  }
  else
  {
    // configuration failed.
  }
  @endcode
 */
return_t 
MSS_MAC_cfg_struct_def_init
(
    MAC_cfg_t * cfg
);

/***************************************************************************//**
  The function MSS_MAC_init() is used to initialize the MAC hardware and 
  driver. This function accepts the pointer to configuration record of type
  MAC_cfg_t. This configuration record must be initialized with proper values
  (either default or custom). Apart from initializing MAC hardware, this function
  initializes the descriptor rings and their pointers to initial values.
  This function enables DMA Rx packet received and Tx packet sent interrupts.
 
  @param cfg
    The cfg parameter is pointer to the MAC configuration structure of MAC_cfg_t
    type. You need to create and initialize this record with valid values either
    by calling MSS_MAC_cfg_struct_def_init() function or by assigning custom 
    values to individual elements of the record. 

  @return
    This function returns 1 upon successful initialization or returns 0 on 
    failure.

  Example:
  @code
    MAC_cfg_t cfg;

    MSS_MAC_cfg_struct_def_init( cfg );

    MSS_MAC_init( cfg );
  @endcode
 */
return_t 
MSS_MAC_init
( 
    MAC_cfg_t * cfg 
);
/***************************************************************************//**
  The function MSS_MAC_alloc_pkt() is to find the next free packet in 
  the given packet containers ring. This function finds the next free packet 
  available for reuse based on the parameter io. A MAC_pkt_t type pointer of 
  the free packet found will be returned.
 
  @param pktring
    The pktring is MAC_pkt_t type packet buffer of size MSS_MAC_TX_RING_SIZE or
    MSS_MAC_RX_RING_SIZE based on whether it is TX or RX packet container.
  
  @param io
    The io parameter tell the function whether the packet ring is of TX or RX.

  @return 
    This function returns MAC_pkt_t type pointer of the free packet available, 
    returns NULL otherwise.

  Example:
  @code
    MAC_pkt_t txpkts[MSS_MAC_TX_RING_SIZE];

    return_t status = MSS_MAC_init_pkt_ring( txpkts, TX );

    if( status == SUCCESS )
        MAC_pkt_t * next_tx_pkt = MSS_MAC_alloc_pkt( txpkts, TX );
  @endcode
 */
MAC_pkt_t *
MSS_MAC_alloc_pkt
(
    MAC_pkt_t *pktring,
    MAC_io_t  io
);

/***************************************************************************//**
  The function MSS_MAC_transfer_rx_pkt() is used to get the received packet to 
  application. This function copies the packet data from the container buffer
  pointed by pkt->buff to the destination buffer pointed by dest parameter only
  when these two pointers are pointing different buffers. If both the dest and
  pkt->buff point the same buffer (in the case user decides to use the same
  packet container buffer for processing the packet) then copy will not be 
  performed. The lock parameter in the packet pointed by pkt is changed to 
  EMPTY indicating that the received packet data is transferred to application.
  This indicates the packet container is free from any received data and can be
  reused or de-allocated.

  NOTE: This function must be called even in the case user decides to use same
        buffer as container for received packet and for processing the received
        packet. In this case the parameters dest and pkt->buff point the same 
        buffer. Calling this function is necessary to change the pkt->lock to 
        EMPTY state which is otherwise not possible.
 
  @param dest
    The dest parameter is pointer to the destination buffer to which the packet
    data needs to be copied.
 
  @param pkt
    The pkt parameter is pointer to the receive packet container in which the 
    received packet will be stored by the driver. 

  @return
    This function returns 1 upon successful initialization or returns 0 on failure.
  Example:
  @code
    MAC_pkt_t rxpkts[MSS_MAC_TX_RING_SIZE];
    uint8_t inbuff[MSS_MAC_RX_BUFF_SIZE];
    return_t status = MSS_MAC_init_pkt_ring( txpkts, RX );

    if( status == SUCCESS )
    {
        MAC_pkt_t * next_rx_pkt = MSS_MAC_get_next_free_pkt( rxpkts, RX );
        
        if( NULL != next_rx_pkt )
        {
            status = MSS_MAC_receive_pkt( next_rx_pkt );

            if( status == SUCCESS )
                while( MSS_MAC_is_pkt_available( next_rx_pkt ) );

            MSS_MAC_transfer_rx_pkt( inbuff, next_rx_pkt);
        }
    }
  @endcode
 */
return_t 
MSS_MAC_transfer_rx_pkt
(
    uint8_t * dest,
    MAC_pkt_t * pkt    
);

/***************************************************************************//**
  The function MSS_MAC_init_pkt_pool() function initializes the packet container
  rings created in application memory space. You need to call this function 
  before using any of the packet container (either TX or RX). This function
  initializes the lock parameter of the TX packet rings to UNLOCK state and RX
  packet rings to EMPTY state, buff parameter to NULL and size parameter to 0.
 
  @param pktring
    This pktring parameter is pointer to the packet buffer that is to be 
    initialized.
  
  @param io
    This io parameter is to mention whether the packet ring is TX or RX. This is 
    of type MAC_io_T.

  @return 
    This function returns 1 on successful execution. Returns 0 otherwise.
  
  Example:
  @code
    MAC_pkt_t txpkts[MSS_MAC_TX_RING_SIZE];

    return_t status = MSS_MAC_init_pkt_pool( txpkts, TX );

    if( status == SUCCESS )
        MAC_pkt_t * next_tx_pkt = MSS_MAC_get_next_free_pkt( txpkts, TX );
  @endcode
 */
return_t 
MSS_MAC_init_pkt_pool
(
    MAC_pkt_t * pktring,
    MAC_io_t io
);

/***************************************************************************//**
  The function MSS_MAC_is_pkt_received() is to find out whether the packet 
  assigned for reception is received and available for use in application. 
  This is function determines availability of the received packet by checking 
  the lock parameter against UNLOCK. If the lock is UNLOCKed then the packet is 
  received and free from driver's possession.
 
  @param pkt
    The pkt is pointer to the MAC_pkt_t type packet which is being checked for 
    availability. 

  @return
    This function returns 1 if the packet is available. Returns 0 if the packet 
    is not available to application.

  Example:
  @code
    MAC_pkt_t rxpkts[MSS_MAC_TX_RING_SIZE];

    return_t status = MSS_MAC_init_pkt_ring( txpkts, RX );

    if( status == SUCCESS )
    {
        MAC_pkt_t * next_rx_pkt = MSS_MAC_get_next_free_pkt( rxpkts, RX );
        
        if( NULL != next_rx_pkt )
        {
            status = MSS_MAC_receive_pkt( next_rx_pkt );

            if( status == SUCCESS )
                while( MSS_MAC_is_pkt_received( next_rx_pkt ) );
        }
    }
  @endcode
 */
return_t 
MSS_MAC_is_pkt_received
(
    MAC_pkt_t * pkt 
);

/***************************************************************************//**
  The function MSS_MAC_is_pkt_transmitted() is to find out whether the packet 
  assigned for transmission is transmitted and available for use in application. 
  This is function determines availability of the received packet by checking 
  the lock parameter against UNLOCK. If the lock is UNLOCKed then the packet is 
  received and free from driver's possession.
 
  @param pkt
    The pkt is pointer to the MAC_pkt_t type packet which is being checked for 
    availability. 

  @return
    This function returns 1 if the packet is available, returns 0 if the packet 
    is not available to application.

  Example:
  @code
    MAC_pkt_t rxpkts[MSS_MAC_TX_RING_SIZE];

    return_t status = MSS_MAC_init_pkt_ring( txpkts, RX );

    if( status == SUCCESS )
    {
        MAC_pkt_t * next_rx_pkt = MSS_MAC_get_next_free_pkt( rxpkts, RX );
        
        if( NULL != next_rx_pkt )
        {
            status = MSS_MAC_receive_pkt( next_rx_pkt );

            if( status == SUCCESS )
                while( MSS_MAC_is_pkt_received( next_rx_pkt ) );
        }
    }
  @endcode
 */
return_t 
MSS_MAC_is_pkt_transmitted
(
    MAC_pkt_t * pkt 
);

/***************************************************************************//**
  The function MSS_MAC_prepare_tx_pkt() is to prepare a packet to be transmitted.
  This function copies the packet data from the buffer pointed by source to 
  the packet container buffer pointed by pkt->buff. If both the source and 
  pkt->buff point the same buffer then copy is not performed. In both cases
  the pkt->lock parameter will be changed to FULL indicating that the packet
  container is full and cannot be reused till the packet is transmitted by 
  the driver. 
  
  Note: This function must be called even though user decides to use the same
  buffer for packet source and to transmit the same packet using pkt->buff 
  container. This is needed because this function changes the pkt->lock 
  parameter to FULL which is verified by MSS_MAC_send_pkt() function to
  successfully transmit the packet. 
 
  @param source
    The source parameter points to the buffer that contains the packet data
    to be transmitted.
 
  @param pkt
    The pkt parameter is the transmit packet container.

  @return
    This function returns 1 on successful execution. Otherwise it returns 0.

  Example:
  @code
    MAC_pkt_t txpkts[MSS_MAC_TX_RING_SIZE];
    uint8_t outbuff[MSS_MAC_TX_BUFF_SIZE];
    return_t status = MSS_MAC_init_pkt_ring( txpkts, TX );

    if( status == SUCCESS )
        MAC_pkt_t * next_tx_pkt = MSS_MAC_get_next_free_pkt( txpkts, TX );

    if( NULL != next_tx_pkt )
    {
        status = MSS_MAC_prepare_tx_pkt( outbuff, next_tx_pkt );
        if( status == SUCCESS )
            MSS_MAC_send_pkt( next_tx_pkt );
    }                         
  @endcode
 */
return_t 
MSS_MAC_prepare_tx_pkt
(
    uint8_t * source,
    MAC_pkt_t * pkt    
);

/***************************************************************************//**
  The function MSS_MAC_read_interface_status() is to read the PHY interface
  status from MII management sub-system of MAC. 
 
  @param intf_status_bit
    The intf_status_bit if of type MAC_intf_status_t enumeration. This parameter
    tells the function what status bit of PHY interface needs to be returned.
 
  @return
    This function returns 1 if the requested status bit is set, returns 0
    otherwise.

  Example:
  @code
    speed = MSS_MAC_read_interface_status( SPEED );
  @endcode
 */
uint8_t
MSS_MAC_read_interface_status
(
    MAC_intf_status_t intf_status_bit    
);

/***************************************************************************//**
  The function MSS_MAC_read_phy_indicator() is used to read the PHY indicators
  which convey information about the current PHY control (read/write/scan)
  operation. This information can be used in implementing PHY interface
  routines.
 
  @param  indicator
    The indicator parameter is the PHY interface status bit to be read. This is
    of type MAC_phy_ind_t.

  @return
    This function returns returns 1 if the indicator is set, returns 0 otherwise.

  Example:
  @code
    if( ! MSS_MAC_read_phy_indicator( BUSY ) )
    {
        MSS_MAC_read_phy_reg(SGMII_PHY_ADDR, MII_BMSR);
    }
  @endcode
 */
uint8_t
MSS_MAC_read_phy_indicator
(
    MAC_phy_ind_t indicator
);

/***************************************************************************//**
  The function MSS_MAC_read_phy_reg() function is used to read the PHY 
  registers which are interfaced to MAC module through MII Management registers.
  This function initiates a read operation for the register addressed in the 
  addressed PHY by using MII management functionality of MAC. This function 
  is used in various PHY interfaces integrated with this driver.
 
  @param phyaddr
    The phyaddr parameter is 5-bit PHY address.
 
  @param regaddr
    The regaddr parameter is 5-bit register address in the above addressed PHY.

  @return 
    This function returns 16-bit value of the register requested to read.

  Example:
  @code
    #include "phy.h"

    void read_phy_status(void)
    {
        uint8_t phy_status = MSS_MAC_read_phy_reg( MII_BMSR );
    }
  @endcode
 */
uint16_t
MSS_MAC_read_phy_reg
(
    uint8_t phyaddr,
    uint8_t regaddr
);

/***************************************************************************//**
  The function MSS_MAC_read_rx_pkt_count() is used to get the count of 
  received packets. This count increments when a packet is successfully 
  received and decrements when a packet is pending for reception i.e by 
  calling MSS_MAC_receive_pkt() function.
 
  @param 
    This function doesn't accept any parameters.

  @return
    This function returns current 8-bit value of received packets count.

  Example:
  @code

    uint8_t pkts_rcvd = MSS_MAC_read_rx_pkt_count();

  @endcode
 */
uint8_t 
MSS_MAC_read_rx_pkt_count
(
    void
);

/***************************************************************************//**
  The function MSS_MAC_read_rx_status() is used to read the MAC's receiver 
  status. It reads the corresponding status bit requested by the caller. 
  Valid receiver status bits are defined by MAC_status_t enumeration. 
 
  @param status_bit
    The status_bit is the receiver status bit whose status is to be read. This
    is of type MAC_status_t. 

  @return
    This function returns 1 if the requested status bit is set, returns 0
    otherwise.

  Example:
  @code
    void custom_rx_isr(void)
    {
        uint8_t  pkt_rcvd = MSS_MAC_read_rx_status( RX_PKT_RCVD );
        if( pkt_rcvd )
        {
            process_pkt();
        }
    }
  @endcode
 */
uint8_t 
MSS_MAC_read_rx_status
(
    MAC_status_t status_bit
);

/***************************************************************************//**
  The function MSS_MAC_read_stat() is  used to read the transmit and receive 
  statistics of Ethernet MAC. This function reads in total 17 receiver 
  statistics, 20 transmitter statistics and 7 frame type statistics as defined
  in MAC_stat_t enumeration. 
 
  @param stat
    The stat parameter is the statistic of type MAC_stat_t.  

  @return
    This function returns 32-bit statistic value requested. But the all
    32-bits in the returned values may not be valid. Based on the statistic
    type requested to read, the returned value will have meaningful bits.

  Example:
  @code
    uint32_t tx_pkts_cnt = MSS_MAC_read_stat( TX_PKT_CNT );
  @endcode
 */
uint32_t 
MSS_MAC_read_stat
(
    MAC_stat_t stat
);

/***************************************************************************//**
  The function MSS_MAC_read_tx_pkt_count() is used to get the count of 
  transmitted packets. This count increments when a packet is successfully 
  transmitted and decrements when a packet is pending for transfer i.e by 
  calling MSS_MAC_send_pkt() function.
 
  @param 
    This function doesn't accept any parameters.

  @return
    This function returns current 8-bit value of transmitted packets count.

  Example:
  @code

    uint8_t pkts_sent = MSS_MAC_read_tx_pkt_count();

  @endcode
 */
uint8_t 
MSS_MAC_read_tx_pkt_count
(
    void
);

/***************************************************************************//**
  The function MSS_MAC_read_tx_status() is used to read the MAC's transmitter 
  status. It reads the corresponding status bit requested by the caller. 
  Valid transmitter status bits are defined by MAC_status_t enumeration. 
 
  @param status_bit
    The status_bit is the transmitter status bit whose status is to be read. This
    is of type MAC_status_t. 

  @return
    This function returns 1 if the requested status bit is set, returns 0
    otherwise.

  Example:
  @code
    void custom_tx_isr(void)
    {
        uint8_t  pkt_sent = MSS_MAC_read_tx_status( TX_PKT_SENT );
        if( pkt_sent )
        {
            send_next_pkt();
        }
    }
 */
uint8_t 
MSS_MAC_read_tx_status
(
    MAC_status_t status_bit
);

/***************************************************************************//**
  The function MSS_MAC_receive_pkt() is used to assign a packet container for 
  reception. When there are no prior packets assigned, the present packet will
  be assigned to first descriptor and the reception is enabled. 

  You should invoke MSS_MAC_alloc_pkt() function with pointer to the packet 
  buffers and RX (MAC_io_t) as arguments. This function call returns the next 
  available packet container that can be used for reception with the lock parameter
  of the packet container marked as EMPTY. The buff parameter in the available 
  packet should be assigned with a buffer that can be used as packet container 
  for reception. For this, it is always advisable to create a separate buffer 
  of size MSS_MAC_MAX_RX_BUF_SIZE.
  
  Then the MSS_MAC_receive_pkt() should be invoked with the packet container in 
  which the frame should be received as the argument. This function only accepts 
  packets which are having the lock element value EMPTY. This function finds the 
  next free transmit descriptor and assigns the packet to the descriptor. It 
  restarts the packet reception if it is halted.
 
  @param pkt
    The pkt parameter is pointer to a packet container of type MAC_pkt_t. The 
    lock element of the packet container should be EMPTY while passed to 
    MSS_MAC_receive_pkt() function. 
 
  @return
    This function returns 1 on successfully assigning the packet to a 
    transmit descriptor, returns 0 otherwise.

  Example:
  @code
    uint8_t rxbuff[MSS_MAC_MAX_RX_BUF_SIZE];
    
    MAC_pkt_t txpkts[MSS_MAC_RX_RING_SIZE];
    
    MAC_pkt_t pkt = MSS_MAC_get_next_free_pkt( rxpkts, RX );
    
    pkt->buff = rxbuff;
    
    MSS_MAC_receive_pkt( pkt );
  @endcode
 */
return_t
MSS_MAC_receive_pkt
(
    MAC_pkt_t * pkt,
    uint8_t  slots
);

/***************************************************************************//**
  The function MSS_MAC_reset() is used to reset the Ethernet MAC hardware and
  driver. This resets MAC transmission and reception control blocks, FIFOs,
  and MII management block in MAC hardware. It resets the transmission and 
  reception descriptors to their initial values and resets the current
  descriptor and next descriptor points to NULL. It is recommended to call
  the MSS_MAC_init() function with default configuration record after reset is
  performed

  NOTE: This function will not perform software reset of the MAC module but
        individually resets different modules of Ethernet MAC.
 
  @param 
  	This function doesn't accept parameters.

  @return
	This function doesn't return a value.
  Example:
  @code
  	MAC_cfg_t cfg;
	MSS_MAC_cfg_struct_def_init( &cfg );

	MSS_MAC_reset();

	MSS_MAC_init( cfg );
  @endcode
 */
void 
MSS_MAC_reset
(
    void
);

/***************************************************************************//**
  The function MSS_MAC_send_pkt() is used to initiate transmission of a packet
  of type MAC_pkt_t. You should invoke MSS_MAC_get_next_free_pkt() function with
  pointer to the packet buffers and TX (MAC_io_t) as arguments. This function call
  returns the next available packet buffer that can be used for transmission. the
  buff parameter in the available packet should be assigned with a buffer that 
  can be used as packet container for transmission. For this, it is always 
  advisable to create a separate buffer of size MSS_MAC_MAX_TX_BUF_SIZE because 
  the it is used for transmission. Then to copy the packet data from source 
  to this packet buffer, you should assign the size of the packet to be 
  transmitted to size element of the packet and call MSS_MAC_prepare_pkt() 
  function with pointer to source and the packet to be transmitted. This function
  copies the data from source buffer to the packet buffer and changes the 
  lock parameter of the packet to FULL.  

  In case the designer decides to use the same packet source buffer as 
  container for transmission, then the buff parameter in the packet should be 
  initialized with the starting address of the packet source buffer. In this
  case invokation of MSS_MAC_prepare_pkt() function will not copy the packet data
  from source to packet container buffer (because the source and packet container
  buffer are same) but simply changes the lock parameter of the packet to FULL.

  Then the MSS_MAC_send_pkt() should be invoked with the packet to be 
  transmitted as the argument. This function only accepts packets which are 
  having the lock element value FULL. This function finds the next free
  transmit descriptor and assigns the packet to the descriptor. It restarts 
  the packet transmission if it is halted.
 
  @param pkt
    The pkt parameter is pointer to a packet container of type MAC_pkt_t. The 
    lock element of the packet container should be FULL while passed to 
    MSS_MAC_send_pkt() function. 
 
  @return
    This function returns 1 on successfully assigning the packet to a 
    transmit descriptor, returns 0 otherwise.

  Example:
  @code
    uint8_t txbuff[MSS_MAC_MAX_TX_BUF_SIZE];
    
    MAC_pkt_t txpkts[MSS_MAC_TX_RING_SIZE];
    
    MAC_pkt_t pkt = MSS_MAC_get_next_free_pkt( txpkts, TX );
    
    pkt->buff = txbuff;
    pkt->size = sizeof(pkt_src_buff);
    
    MSS_MAC_prepare_pkt( pkt_src_buff, pkt );
    
    MSS_MAC_send_pkt( pkt );
  @endcode
 */
return_t 
MSS_MAC_send_pkt
(
    MAC_pkt_t * pkt,
    uint8_t  slots
);

/***************************************************************************//**
  The function MSS_MAC_set_mac_addr() is used to assign station's MAC address.
  At the time of initialization, the driver assigns a default MAC address 
  defined as MSS_MAC_DEFAULT_MAC_ADDRESS. During system initialization, if 
  user prefers to change the MAC address, then this function can be used.
   
  @param address
	The address parameter points to the byte buffer that contians the six 
	byte MAC address that needs to be assigned. Only first six bytes of 
	the address pointed by this parameter will be used to assign new
	MAC address.
 
  @return
	This function returns 1 on successful assignment of MAC address or 
	return 0 on failure.

  Example:
  @code
  	uint8_t *mac = {0xCA, 0xCB, 0xCC, 0xCD, 0xCE, 0xCF};
	MSS_MAC_set_mac_addr( mac );
  @endcode
 */
return_t 
MSS_MAC_set_mac_addr
(
    uint8_t *address
);

/***************************************************************************//**
  The function MSS_MAC_write_phy_reg() is to write a 16-bit value to PHY
  registers. The PHY address and register address comply RFC802.3 clause 22.
  This function is used in various MDD configuration functions to advertise
  PHY capabilities and so on.
 
  @param phyaddr
	The phyaddr parameter is a 5-bit PHY address as per 802.3 RFC. 
	The MSS_MAC_write_phy_reg() function checks this parameter to comply the
	standard.
 
  @param regaddr
	The regaddr parameter is a 5-bit PHY register address as per 802.3 RFC. 
	The MSS_MAC_write_phy_reg() function checks this parameter to comply the
	standard.

  @param regval
    The regval parameter is 16-bit value that is to be written in addressed PHY
    register.

  @return
	This function returns 1 on successful execution, returns 0 otherwise.

  Example:
  @code
    #include "mss_ethernet_sgmii_phy.h"
    #include "phy.h"

    void rest_sgmii_phy(void)
    {
  	    MSS_MAC_write_phy_reg( SGMII_PHY_ADDR, MII_BMCR, 0x8000 );
    }
  @endcode
 */
return_t 
MSS_MAC_write_phy_reg
(
    uint8_t phyaddr,
    uint8_t regaddr,
    uint16_t regval
);

/***************************************************************************//**
  The function MSS_MAC_enable_irq() is used to enable EthernetMAC interrupts. 
  Any one or a combination of interrupts can be enabled using this function.
 
  @param irq
    The irq parameter is the bit mask of the interrupt to be enabled. It can 
    be any one or a combination of following definitions.
            MSS_MAC_TXPKTSENT_IRQ    = bit mask 1
            MSS_MAC_TXUNDRRUN_IRQ    = bit mask 2
            MSS_MAC_TXBUSERR_IRQ     = bit mask 8
            MSS_MAC_RXPKTRCVD_IRQ    = bit mask 16
            MSS_MAC_RXOVRFLOW_IRQ    = bit mask 64
            MSS_MAC_RXBUSERR_IRQ     = bit mask 128

  @return
    This function doesn't return a value.

  Example:
  @code
    MSS_MAC_enable_irq(MSS_MAC_TXPKTSENT_IRQ | MSS_MAC_RXPKTRCVD_IRQ);
  @endcode
 */
void
MSS_MAC_enable_irq
(
    uint8_t irq
);

/***************************************************************************//**
  The function MSS_MAC_disable_irq() is used to disable Ethernet MAC interrupts. 
  Any one or a combination of interrupts can be disabled using this function.
 
  @param irq
    The irq parameter is the bit mask of the interrupt to be disabled. It can 
    be any one or a combination of following definitions.
            MSS_MAC_TXPKTSENT_IRQ               (1u<<0)
            MSS_MAC_TXUNDRRUN_IRQ               (1u<<1)
            MSS_MAC_TXBUSERR_IRQ                (1u<<3)
            MSS_MAC_RXPKTRCVD_IRQ               (1u<<4)
            MSS_MAC_RXOVRFLOW_IRQ               (1u<<6)
            MSS_MAC_RXBUSERR_IRQ                (1u<<7)

  @return
    This function doesn't return a value.

  Example:
  @code
    MSS_MAC_disable_irq(MSS_MAC_TXPKTSENT_IRQ | MSS_MAC_RXPKTRCVD_IRQ);
  @endcode
 */
void
MSS_MAC_disable_irq
(
    uint8_t irq
);

/***************************************************************************//**
  The function MSS_MAC_start_rx() is used to start the reception both at DMA
  and MAC levels. This function should be usually called when EthernetMAC's 
  reception is already stopped. This function call will be ineffective if the
  Ethernet MAC reception is already active. 
 
  @param 
    This function doesn't accept any parameters.  

  @return
    This function doesn't return a value.    

  Example:
  @code
    MSS_MAC_start_rx();
  @endcode
 */
void
MSS_MAC_start_rx
(
    void
);

/***************************************************************************//**
  The function MSS_MAC_start_tx() is used to start the transmission both at DMA
  and MAC levels. This function should be ususally called when EthernetMAC's 
  transmission is already stopped. This function call will be ineffective if the
  Ethernet MAC transmission is already active. 
 
  @param 
    This function doesn't accept any parameters.  

  @return
    This function doesn't return a value.    

  Example:
  @code
    MSS_MAC_start_tx();
  @endcode
 */
void
MSS_MAC_start_tx
(
    void
);

/***************************************************************************//**
  The function MSS_MAC_stop_rx() is used to stop the reception both at DMA
  and MAC levels. After calling this function, no more "Packet received" 
  interrupts will be generated.
 
  @param 
    This function doesn't accept any parameters. 

  @return
    This function doesn't return a value.

  Example:
  @code
    MSS_MAC_stop_rx();
  @endcode
 */
void
MSS_MAC_stop_rx
(
    void
);

/***************************************************************************//**
  The function MSS_MAC_stop_tx() is used to stop the transmission both at
  DMA and MAC levels. After calling this function, no more "transmission packet 
  sent" interrupts will be generated. 
 
  @param 
    This function doesn't accept any parameters.

  @return
    This function doesn't return a value.

  Example:
  @code
    MSS_MAC_stop_tx();
  @endcode
 */
void
MSS_MAC_stop_tx
(
    void
);

/***************************************************************************//**
  The function MSS_MAC_assign_irq_handler() is used to assign a custorm 
  interrupt handler for any of the MAC interrupts. Different MAC interrupts are
  available by the MAC_irq_t type enumeration. It is always prescribed to use 
  MAC_lock_t type to change the state of the "lock" parameter of packet containers
  in the custom interrupt handlers.
 
  @param irqnum
    The irqnum parameter is the MAC interrupt number for which the custom handler
    should be assigned.
  
  @param callback
    The callback parameter is the custom handler that will be assigned to the
    interrupt pointed by the irqnum.

  @return
    This function returns 1 on successful execution, returns 0 otherwise.

  Example:
  @code
    MSS_MAC_assign_irq_handler( MSS_MAC_TXPKTSENT_IRQ, my_tx_callback );
  @endcode
 */
return_t 
MSS_MAC_assign_irq_handler
(
    MAC_irq_t irqnum,
    MAC_callback_t callback    
);


#ifdef __cplusplus
}
#endif

#endif /* MSS_ETHERNET_MAC_H_ */
