/*******************************************************************************
 * (c) Copyright 2011 Actel Corporation.  All rights reserved.
 *
 * G4Main Ethernet MAC SGMII PHY interface firmware
 *
 * SVN $Revision: 3526 $
 * SVN $Date: 2011-04-22 14:53:25 +0530 (Fri, 22 Apr 2011) $
 */
#ifdef __cplusplus
extern "C" {
#endif

#include "phy.h"
#include "mss_ethernet_mac_types.h"
#include "mss_ethernet_mac.h"
#include "mss_ethernet_sgmii_phy.h"

/**************************************************************************/
/* Preprocessor Macros                                                    */
/**************************************************************************/
/*
 * SGMII status register: Bit 6
 * Indicates that PHY is capable of detecting frames from MII with 
 * preamble supressed.
 */
#define SGMII_PREAM_SUPRESS_ENABLE          (1u<<6)

/*
 * SGMII extended status bits
 */
#define BASE1000X_FULL_DUPLEX               (1u<<15)
#define BASE1000X_HALF_DUPLEX               (1u<<14)
#define BASE1000T_FULL_DUPLEX               (1u<<13)
#define BASE1000T_HALF_DUPLEX               (1u<<12)


/**************************************************************************/
/* Private Function declarations                                          */
/**************************************************************************/





/**************************************************************************/
/* Public Function definitions                                            */
/**************************************************************************/

/**************************************************************************//**
 * See mss_ethernet_sgmii_phy.h for details of how to use this function.
 */
return_t 
sgmii_init
(
    MAC_cfg_t * cfg
)
{
    return_t status = FAILED;
    uint32_t temp;

    /* Reset the PHY interface */
    temp = MSS_MAC_read_phy_reg(SGMII_PHY_ADDR, MII_BMCR);
	temp |= BMCR_RESET;
	status = MSS_MAC_write_phy_reg(SGMII_PHY_ADDR, MII_BMCR, temp);

    /* No need to check cfg->interface == SGMII here
       as this function will be called after detection SGMII 
       request in the user configuration */

    /* Disable loopback */
    /* Enable auto negotiation and restart auto negotiation */

    temp = ~BMCR_LOOPBACK;
    if( cfg->auto_negotiation == MSS_MAC_AUTO_NEGOTIATION_ENABLE)
        temp |= (BMCR_ANENABLE | BMCR_ANRESTART);
    status = status && MSS_MAC_write_phy_reg(SGMII_PHY_ADDR, MII_BMCR, temp);
    
    return status;
}

/**************************************************************************//**
 * See mss_ethernet_sgmii_phy.h for details of how to use this function.
 */
return_t 
sgmii_config_aneg
(
    MAC_cfg_t * cfg
)
{
    return_t status = FAILED;
    uint16_t temp ;
    uint32_t timeout = 100000;

    /* Advertise */
    temp = AN_ADVT_RESERVED_DEF_VAL;
    if(cfg->fullduplex == MSS_MAC_FULLDUPLEX_ENABLE)
        temp |= AN_ADVT_FULL_DUPLEX;
    else
        temp &= ~AN_ADVT_FULL_DUPLEX;
    switch(cfg->speed)
    {
        case MAC10MBPS : temp |= AN_ADVT_10MBPS; break; 
        case MAC100MBPS: temp |= AN_ADVT_100MBPS; break;
        case MAC1000MBPS: temp |= AN_ADVT_1000MBPS; break;
        default: break;
    }
    /* for integration with MCXMAC : link-up = 0 */
    temp &= ~AN_ADVT_LINKUP;
    
    /* Advertise the configured capabilities */
    status = MSS_MAC_write_phy_reg(SGMII_PHY_ADDR, SGMII_AN_ADVT, temp);

    if(0 == status )
        return status;

    /* disable loopback and restart auto negotiation */
    temp = ~BMCR_LOOPBACK;
    if( cfg->auto_negotiation == MSS_MAC_AUTO_NEGOTIATION_ENABLE)
        temp |= (BMCR_ANENABLE | BMCR_ANRESTART);

    status = MSS_MAC_write_phy_reg(SGMII_PHY_ADDR, MII_BMCR, temp);

    /* Wait till auto-negotiation is completed or timed-out */
    do
    {
        temp = MSS_MAC_read_phy_reg(SGMII_PHY_ADDR, MII_BMSR);
        --timeout;
    }while( timeout && !(temp & BMSR_ANEGCOMPLETE));

    /* If not timedout */
    if(timeout)
        status = SUCCESS;

    return status;
}

/**************************************************************************//**
 * See mss_ethernet_sgmii_phy.h for details of how to use this function.
 */
return_t 
sgmii_read_status
(
    MAC_cfg_t * cfg
)
{
    return_t status = FAILED;
    uint32_t temp = 0;

    temp = MSS_MAC_read_phy_reg(SGMII_PHY_ADDR, MII_BMSR);

    /* Read the status only if no remote fault is detected,
       auto-negotiation is completed and the link is UP */
    if( (temp & BMSR_LSTATUS) && (temp & BMSR_ANEGCOMPLETE) &&
        !(temp & BMSR_RFAULT) )
    {
        temp = MSS_MAC_read_phy_reg(SGMII_PHY_ADDR, MII_BMSR);

        /* Go ahead only if link between SGMII and PHY is UP */
        if( (temp & AN_BPABLTY_LINKUP) )
        {
            if( temp & AN_BPABLTY_FULL_DUPLEX)       
                cfg->fullduplex = MSS_MAC_FULLDUPLEX_ENABLE;
            else
                cfg->fullduplex = MSS_MAC_FULLDUPLEX_DISABLE;
    
            switch( temp & AN_BPABLTY_LINK_SPEED)
            {
                case AN_ADVT_10MBPS  : cfg->speed = MAC10MBPS; break;
                case AN_ADVT_100MBPS : cfg->speed = MAC100MBPS;break;
                case AN_ADVT_1000MBPS: cfg->speed = MAC1000MBPS;break;
                default: cfg->speed = INVALID_SPEED;
            }
            /* Comply with extended status */
            temp = MSS_MAC_read_phy_reg(SGMII_PHY_ADDR, MII_BMSR);

            if( temp & BASE1000X_FULL_DUPLEX )
            {
                cfg->fullduplex = MSS_MAC_FULLDUPLEX_ENABLE;
                cfg->speed = MAC1000MBPS;
            }
            if( temp & BASE1000X_HALF_DUPLEX )
            {
                cfg->fullduplex = MSS_MAC_FULLDUPLEX_DISABLE;
                cfg->speed = MAC1000MBPS;
            }
            if( temp & BASE1000T_FULL_DUPLEX )
            {
                cfg->fullduplex = MSS_MAC_FULLDUPLEX_ENABLE;
                cfg->speed = MAC1000MBPS;
            }
            if( temp & BASE1000T_HALF_DUPLEX )
            {
                cfg->fullduplex = MSS_MAC_FULLDUPLEX_DISABLE;
                cfg->speed = MAC1000MBPS;
            }
            status = SUCCESS;
        }
    }

    return status;
}

#ifdef __cplusplus
}
#endif

/******************************** END OF FILE ******************************/






