/*******************************************************************************
 * (c) Copyright 2012 Microsemi Corporation.  All rights reserved.
 *
 * Microsemi Smartfusion2 USB device logical driver layer Implementation.	
 *  
 *
 * This file implements the common functinality of USB logical device layer, 
 * which includes Initialization of core in device mode, USB standard request 
 * handling, distribution of requests to specific class, interface or endpoints.
 *
 * SVN $Revision: 5027 $
 * SVN $Date: 2013-02-01 19:06:39 +0530 (Fri, 01 Feb 2013) $
 */

#include "../../CMSIS/mss_assert.h"
#include "mss_usb_config.h"
#include "mss_usb_device.h"
#include "mss_usb_common_cif.h"
#include "mss_usb_device_cif.h"
#include <stdio.h>

#ifdef __cplusplus
extern "C" {
#endif

#define SETUP_PKT_INIT();  	                        setup_pkt.request_type = 0;\
													setup_pkt.request = 0;\
													setup_pkt.value = 0;\
													setup_pkt.index = 0;\
													setup_pkt.length = 0;

/******************************************************************************
* Exported instances.
*/                                                    
/*
 This structure should be implemented for bare testing of the USB transfers,
 when in the test mode. No other class specific components or enumeration
 related APIs are called.
 This mode is only provided for internal customers for testing purpose.
*/

#ifdef MSS_USB_DEVICE_TEST_MODE
extern mss_usb_user_device_test_cb_t usb_device_test_cb;
#endif

/******************************************************************************
* Global variables used within this file.
*/

/* This structure must be implemented by user to provide the descriptors. */
mss_usb_user_descriptors_cb_t *g_usb_user_descriptors_cb;

/* This structure must be implemented by class specific component.*/
mss_usb_class_cb_t *g_usb_class_cb;

uint8_t  g_cmd_code = 0xFFu;
uint16_t g_cmd_len  = 0u;

uint16_t g_device_status;
mss_usb_core_info_t core_info;
mss_usb_device_t device_config;

/* This structure is used to hold the setup packet received on control endpoint*/
mss_usb_device_setup_pkt_t setup_pkt;

mss_usb_ep_t device_ep[5];
mss_usb_ep_t device_rx_ep[5];

uint8_t g_cep_data_buf[64];
uint32_t g_cep_data_buf_len;

/******************************************************************************
 Private function declarations for this file (USB Device Core Driver component).
*/
static void MSS_USB_device_cep_setup_cb(uint8_t status);
static void MSS_USB_device_cep_dataout_cb(uint8_t status);
static void MSS_USB_device_cep_datain_cb(uint8_t status);
static void MSS_USB_device_reset_cb(void);
static void MSS_USB_device_sof_cb(uint8_t status);
static void MSS_USB_device_suspend_cb(void);
static void MSS_USB_device_resume_cb(void);
static void MSS_USB_device_ep_dataout_cb(mss_usb_ep_num_t num, uint8_t status);
static void MSS_USB_device_ep_datain_cb(mss_usb_ep_num_t num, uint8_t status);
static uint8_t MSS_USB_class_requests(uint8_t** buf_pp, uint32_t* len_p);
static uint8_t MSS_USB_vendor_requests(uint8_t** buf_pp, uint32_t* len_p);
static uint8_t MSS_USB_get_descriptor(uint8_t** buf_pp, uint32_t* len_p);
static uint8_t MSS_USB_set_address(void);
static uint8_t MSS_USB_set_config(void);
static uint8_t MSS_USB_get_config(uint8_t** buf_pp,	uint32_t* len_p);
static uint8_t MSS_USB_get_status(uint8_t** buf_pp,	uint32_t* len_p);
static uint8_t MSS_USB_set_feature(void);
static uint8_t MSS_USB_clr_feature(void);
static uint8_t MSS_USB_set_descriptor(void);
static uint8_t MSS_USB_std_requests(uint8_t** buf,uint32_t* length);

/*
  This structure implements the callback functions to be called on respective 
  USB interrupts.
*/
mss_usb_device_cb_t g_mss_usb_device_cb =
{
    MSS_USB_device_ep_dataout_cb,
    MSS_USB_device_ep_datain_cb,

	MSS_USB_device_cep_setup_cb,
    MSS_USB_device_cep_dataout_cb,
    MSS_USB_device_cep_datain_cb,

	MSS_USB_device_sof_cb,
	MSS_USB_device_reset_cb,
    MSS_USB_device_suspend_cb,
	MSS_USB_device_resume_cb
};

/******************************************************************************
  Exported functions from this file (USB Device Core Driver component)
*/

/******************************************************************************
  See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_init
(
	mss_usb_device_speed_t speed
)
{
	device_config.device_addr = 0x00u;
	device_config.device_total_interfaces = 0x00u;
	device_config.device_total_ep = 0x00u;
	device_config.device_state = MSS_USB_NOT_ATTACHED_STATE;
	device_config.device_speed = speed;

	device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;

    MSS_USB_core_device_init(device_config.device_speed);

    MSS_USB_core_device_connect();
	
    device_config.device_state = MSS_USB_POWERED_STATE;
}

/******************************************************************************
  See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_set_desc_cb_handler
(
    mss_usb_user_descriptors_cb_t* user_desc_cb
)
{
    g_usb_user_descriptors_cb = user_desc_cb;
}
/******************************************************************************
  See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_set_class_cb_handler
(
    mss_usb_class_cb_t* class_cb
)
{
    g_usb_class_cb = class_cb;
}
/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_cep_error
(
	void
)
{
	MSS_USB_core_cep_stall();
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_tx_ep_error
(
	mss_usb_ep_num_t ep_num
)
{
	MSS_USB_core_tx_ep_stall(ep_num);
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_rx_ep_error
(
	mss_usb_ep_num_t ep_num
)
{
	MSS_USB_core_rx_ep_stall(ep_num);
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_cep_clr_error
(
	void
)
{
	MSS_USB_core_cep_clr_stall();
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_tx_ep_clr_error
(
	mss_usb_ep_num_t ep_num
)
{
	MSS_USB_core_tx_ep_clr_stall(ep_num);
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_rx_ep_clr_error
(
	mss_usb_ep_num_t ep_num
)
{
	MSS_USB_core_rx_ep_clr_stall(ep_num);
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_cep_configure
(
	void
)
{
	device_ep[MSS_USB_CTRL_EP].num	= MSS_USB_CTRL_EP;
	device_ep[MSS_USB_CTRL_EP].stall = 0u;
	device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
	device_ep[MSS_USB_CTRL_EP].xfr_type = MSS_USB_XFR_CONTROL;

	/*FIFO arress	*/
	device_ep[MSS_USB_CTRL_EP].buf_addr = 0u;

	device_ep[MSS_USB_CTRL_EP].max_pkt_size = CTRL_MAX_PKT_SIZE;
	device_ep[MSS_USB_CTRL_EP].txn_length = SETUP_PKT_SIZE;

	MSS_USB_core_device_cep_configure();
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_cep_read_prepare
(
	uint8_t * addr,
	uint32_t length
)
{
	device_ep[MSS_USB_CTRL_EP].buf_addr = addr;
	device_ep[MSS_USB_CTRL_EP].xfr_length = length;
	device_ep[MSS_USB_CTRL_EP].xfr_count = 0u;
	device_ep[MSS_USB_CTRL_EP].txn_count = 0u;

	if(device_ep[MSS_USB_CTRL_EP].xfr_length > device_ep[MSS_USB_CTRL_EP].max_pkt_size)
	{
		device_ep[MSS_USB_CTRL_EP].txn_length = device_ep[MSS_USB_CTRL_EP].max_pkt_size;
	}
	else
	{
		device_ep[MSS_USB_CTRL_EP].txn_length = length;
	}
	MSS_USB_core_device_cep_rx_prepare(&device_ep[MSS_USB_CTRL_EP]);
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_cep_write
(
	uint8_t * addr,
	uint32_t length
)
{
	device_ep[MSS_USB_CTRL_EP].buf_addr = addr;
	device_ep[MSS_USB_CTRL_EP].xfr_length = length;

	device_ep[MSS_USB_CTRL_EP].xfr_count = 0u;
	device_ep[MSS_USB_CTRL_EP].txn_count = 0u;

	if(device_ep[MSS_USB_CTRL_EP].xfr_length > device_ep[MSS_USB_CTRL_EP].max_pkt_size)
	{
		device_ep[MSS_USB_CTRL_EP].txn_length = device_ep[MSS_USB_CTRL_EP].max_pkt_size;
	}
	else
	{
		device_ep[MSS_USB_CTRL_EP].txn_length = length;
	}
	MSS_USB_core_device_cep_write_pkt(&device_ep[MSS_USB_CTRL_EP]);
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_tx_ep_configure
(
	mss_usb_ep_num_t ep_num,
	uint16_t fifo_addr,
	uint16_t fifo_size,		/*This is actual size of FIFO independent of DPB*/
	uint16_t max_pkt_size,
	uint8_t num_usb_pkt,
	uint8_t dma_enable,
	mss_usb_dma_channel_t dma_channel,
	mss_usb_xfr_type_t xfr_type,
    uint32_t add_zlp
)
{
	uint8_t err_check = SUCCESS;
    uint16_t std_max_pkt_sz = USB_HS_BULK_MAX_PKT_SIZE;

	if(MSS_USB_DEVICE_HS == device_config.device_speed)
	{
		switch(xfr_type)
		{
			case MSS_USB_XFR_BULK:
				std_max_pkt_sz = USB_HS_BULK_MAX_PKT_SIZE;
				break;
			case MSS_USB_XFR_INTERRUPT:
				std_max_pkt_sz = USB_HS_INTERRUPT_MAX_PKT_SIZE;
				break;
			case MSS_USB_XFR_ISO:
				std_max_pkt_sz = USB_HS_ISO_MAX_PKT_SIZE;
				break;
			default:
				err_check = FAIL;
		}
	}
	else if(MSS_USB_DEVICE_FS == device_config.device_speed)
	{
		switch(xfr_type)
		{
			case MSS_USB_XFR_BULK:
				std_max_pkt_sz = USB_FS_BULK_MAX_PKT_SIZE;
				break;
			case MSS_USB_XFR_INTERRUPT:
				std_max_pkt_sz = USB_FS_INTERRUPT_MAX_PKT_SIZE;
				break;
			case MSS_USB_XFR_ISO:
				std_max_pkt_sz = USB_FS_ISO_MAX_PKT_SIZE;
				break;
			default:
				err_check = FAIL;
		}
	}

	if(max_pkt_size > std_max_pkt_sz)
	{
		err_check = FAIL;
	}
	if(fifo_size < max_pkt_size)
	{
		err_check = FAIL;
	}

	ASSERT(err_check == SUCCESS);

	if(err_check == SUCCESS)
	{
		if((max_pkt_size * 2) <= fifo_size)
		{
			device_ep[ep_num].dpb_enable = DPB_ENABLE;
		}
		else
		{
			device_ep[ep_num].dpb_enable = DPB_DISABLE;
		}
        
        device_ep[ep_num].num = ep_num;
        device_ep[ep_num].fifo_addr = fifo_addr;
        device_ep[ep_num].fifo_size = fifo_size;
        device_ep[ep_num].max_pkt_size = max_pkt_size;
        device_ep[ep_num].num_usb_pkt = num_usb_pkt;
        device_ep[ep_num].dma_enable = dma_enable; 	
        device_ep[ep_num].dma_channel = dma_channel;
        device_ep[ep_num].stall = 0;
        device_ep[ep_num].state = MSS_USB_EP_VALID;
        device_ep[ep_num].xfr_type = xfr_type;
        device_ep[ep_num].buf_addr = 0u;
        device_ep[ep_num].add_zlp = add_zlp;
        MSS_USB_core_device_tx_ep_configure(&device_ep[ep_num]);
	}
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_rx_ep_configure
(
	mss_usb_ep_num_t ep_num,
	uint16_t fifo_addr,
	uint16_t fifo_size,		/*This is actual size of FIFO independent of DPB */
	uint16_t max_pkt_size,
	uint8_t num_usb_pkt,
	uint8_t dma_enable,
	mss_usb_dma_channel_t dma_channel,
	mss_usb_xfr_type_t xfr_type,
    uint32_t add_zlp
)
{
	uint8_t err_check = SUCCESS;
	uint16_t std_max_pkt_sz = USB_HS_BULK_MAX_PKT_SIZE;

	if(MSS_USB_DEVICE_HS == device_config.device_speed)
	{
		switch(xfr_type)
		{
			case MSS_USB_XFR_BULK:
				std_max_pkt_sz = USB_HS_BULK_MAX_PKT_SIZE;
				break;
			case MSS_USB_XFR_INTERRUPT:
				std_max_pkt_sz = USB_HS_INTERRUPT_MAX_PKT_SIZE;
				break;
			case MSS_USB_XFR_ISO:
				std_max_pkt_sz = USB_HS_ISO_MAX_PKT_SIZE;
				break;
			default:
				err_check = FAIL;
		}
	}
	else if(MSS_USB_DEVICE_FS == device_config.device_speed)
	{
		switch(xfr_type)
		{
			case MSS_USB_XFR_BULK:
				std_max_pkt_sz = USB_FS_BULK_MAX_PKT_SIZE;
				break;
			case MSS_USB_XFR_INTERRUPT:
				std_max_pkt_sz = USB_FS_INTERRUPT_MAX_PKT_SIZE;
				break;
			case MSS_USB_XFR_ISO:
				std_max_pkt_sz = USB_FS_ISO_MAX_PKT_SIZE;
				break;
			default:
				err_check = FAIL;
		}
	}

	if(max_pkt_size > std_max_pkt_sz)
	{
		err_check = FAIL;
	}
	if(fifo_size < max_pkt_size)
	{
		err_check = FAIL;
	}

	ASSERT(err_check == SUCCESS);

	if(err_check == SUCCESS)
	{
		if((max_pkt_size * 2 ) <= fifo_size)
		{
			device_rx_ep[ep_num].dpb_enable = DPB_ENABLE;
		}
		else
		{
			device_rx_ep[ep_num].dpb_enable = DPB_DISABLE;
		}
        device_rx_ep[ep_num].num = ep_num;
        device_rx_ep[ep_num].fifo_addr = fifo_addr;
        device_rx_ep[ep_num].fifo_size = fifo_size;
        device_rx_ep[ep_num].max_pkt_size = max_pkt_size;
        device_rx_ep[ep_num].num_usb_pkt = num_usb_pkt;
        device_rx_ep[ep_num].dma_enable = dma_enable; 	
        device_rx_ep[ep_num].dma_channel = dma_channel;
        device_rx_ep[ep_num].stall = 0;
        device_rx_ep[ep_num].state = MSS_USB_EP_VALID;
        device_rx_ep[ep_num].xfr_type = xfr_type;
        device_rx_ep[ep_num].buf_addr = 0u;
        device_rx_ep[ep_num].add_zlp = add_zlp;
        
        MSS_USB_core_device_rx_ep_configure(&device_rx_ep[ep_num]);
    }
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_rx_ep_read_prepare
(
	mss_usb_ep_num_t ep_num,
	uint8_t * addr,
	uint32_t length
)
{
	device_rx_ep[ep_num].buf_addr = addr;
	device_rx_ep[ep_num].xfr_length = length; 	
	device_rx_ep[ep_num].xfr_count = 0u;	
	device_rx_ep[ep_num].txn_count = 0u;	

	/*ASSERT when length is 0, address is null or ep number is wrong*/
	if(length > 0)
	{
		/*
        TODO:	While using DMA, RxMaxP payload value MUST be an even number 
        when dealing with Multi packet Bulk transaction for
		proper interrupt generation. This needs to be taken care.
        */
		/*
		   section 8.4.2.3 -- No support for HB in Interrupt transfers. 
           i.e. maximum one packet(num_usb_pkt=1)can be transffered in one frame.
		*/
		if(length >= device_rx_ep[ep_num].max_pkt_size)
		{	
			device_rx_ep[ep_num].txn_length = device_rx_ep[ep_num].max_pkt_size;							
		}
		else
		{
			device_rx_ep[ep_num].txn_length = length;
		}
	}
    
	MSS_USB_core_device_rx_ep_read_prepare(&device_rx_ep[ep_num]);
}

/***************************************************************************//**
 See mss_usb_device.h for details of how to use this function.
*/
void 
MSS_USB_device_tx_ep_write
(
	mss_usb_ep_num_t ep_num,
	uint8_t * addr,
	uint32_t length
)
{
	uint8_t dpb = 1u;
	
    /* assert if uneven value for BULK transfers */
	ASSERT(ep_num);
	ASSERT(addr != 0);
    
	if((ep_num) && (addr != 0))
	{	
		if(device_ep[ep_num].dpb_enable == DPB_ENABLE)
		{
			dpb = 2u;
		}
		device_ep[ep_num].num = ep_num;
		device_ep[ep_num].buf_addr = addr;
		device_ep[ep_num].xfr_length = length; 	
		device_ep[ep_num].xfr_count = 0u; 
		device_ep[ep_num].txn_count = 0u; 
								
		if(device_ep[ep_num].xfr_type == MSS_USB_XFR_BULK)
		{
			/*
			TODO: TxMaxP payload value MUST be an even number when dealing with 
    	    Multi packet Bulk transaction for proper interrupt generation. 
	       	This needs to be taken care.
			*/
			if(length <= device_ep[ep_num].fifo_size)			
			{
				device_ep[ep_num].txn_length = length;				
			}
			else 
			{
				device_ep[ep_num].txn_length= device_ep[ep_num].fifo_size;				
			}
		}

		/*
        section 8.4.2.3 -- No support for HB in Interrupt transfers.
        i.e. maximum one packet(num_usb_pkt=1)can be transffered in one frame.
		*/
		else if(device_ep[ep_num].xfr_type == MSS_USB_XFR_INTERRUPT)
		{
			if(length >= (device_ep[ep_num].max_pkt_size * dpb))
			{	
				device_ep[ep_num].txn_length = (device_ep[ep_num].max_pkt_size * dpb);							
			}
			else
			{
				device_ep[ep_num].txn_length = length;
			}
		}
		else if(device_ep[ep_num].xfr_type == MSS_USB_XFR_ISO)
		{
			if(length >= (device_ep[ep_num].max_pkt_size * dpb))
			{	
				device_ep[ep_num].txn_length = (device_ep[ep_num].max_pkt_size * dpb);							
			}
			else
			{
				device_ep[ep_num].txn_length = length;
			}
		}
		MSS_USB_core_ep_write_pkt(&device_ep[ep_num]);
	}
	else
	{
     	ASSERT(0);
	}
}

/***************************************************************************//**
 This call-back function is executed on receiving SOF interrupt in Device mode.
*/
static void
MSS_USB_device_sof_cb
(
	uint8_t status
)
{
#ifndef MSS_USB_DEVICE_TEST_MODE
#else
	usb_device_test_cb.usb_user_device_test_sof(status);	
#endif

}
/******************************************************************************
 This Callback function is executed when reset interrupt is received when 
 the core is in device mode.
*/
static void 
MSS_USB_device_reset_cb
(
	void
)
{
#ifndef MSS_USB_DEVICE_TEST_MODE
	/*
	TODO: After reset, we can confirm if HS mode is negortiated by using 
	MSS_USB_core_is_hs_mode()
	*/

	device_config.device_addr = 0x00u;
	device_config.device_total_interfaces = 0x00u;
	device_config.device_total_ep = 0x00u;
	device_config.device_state = MSS_USB_DEFAULT_STATE;

    device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
	device_config.remote_wakeup = 0x00u;

	SETUP_PKT_INIT();
	
	MSS_USB_device_cep_configure();
	MSS_USB_device_cep_read_prepare((uint8_t*)&setup_pkt, 8u);
#else
	usb_device_test_cb.usb_user_device_test_reset();
#endif	

}

/******************************************************************************
 This Callback function is executed when Setup packet is received on Control EP .
*/
static void 
MSS_USB_device_cep_setup_cb
(
	uint8_t status
)
{
#ifndef MSS_USB_DEVICE_TEST_MODE
	uint8_t* buf;
	uint32_t length;
	uint8_t result = FAIL;

    if(status)
	{
        /*
		EP0 was previously stalled, clear the error condition and 
		prepare for next transaction
		*/
        device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
        
        SETUP_PKT_INIT();
        
        MSS_USB_device_cep_read_prepare((uint8_t*)&setup_pkt, 8u);
        g_cmd_code = 0xFFu;
        g_cep_data_buf_len = 0u;
	}
	else
	{
		MSS_USB_core_device_cep_read_pkt(&device_ep[MSS_USB_CTRL_EP]);

		if(device_ep[MSS_USB_CTRL_EP].txn_length == SETUP_PKT_SIZE)
		{
			device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_SETUP;
            
			if(USB_STANDARD_REQUEST == (setup_pkt.request_type & USB_STD_REQ_TYPE_MASK))
			{
				result = MSS_USB_std_requests(&buf, &length);
			}
			else if(USB_CLASS_REQUEST == (setup_pkt.request_type & USB_STD_REQ_TYPE_MASK))
			{
				result = MSS_USB_class_requests(&buf, &length);
			}
			else if(USB_VENDOR_REQUEST == (setup_pkt.request_type & USB_STD_REQ_TYPE_MASK))
			{
				result = MSS_USB_vendor_requests(&buf, &length);
			}
		}
		else
		{
		    ASSERT(0);
		}
        if(result)
        {
			if(setup_pkt.length == 0u)
			{
				SETUP_PKT_INIT();
        		device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
				MSS_USB_device_cep_read_prepare((uint8_t*)&setup_pkt,8u);
			}	
			else 
			{
				if((setup_pkt.request_type & USB_STD_REQ_DATA_DIR_MASK))
				{
					device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_DATAIN;
                    /*dataend will be taken care by DataIn_CB*/
					MSS_USB_device_cep_write(buf,length);	
				}
				else
				{
					device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_DATAOUT;
                    
                    /*dataend will be taken care by DataIn_CB*/
					MSS_USB_device_cep_read_prepare(g_cep_data_buf,
                                                    g_cep_data_buf_len); 
				}
			}
        }
        else
        {
			device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
            SETUP_PKT_INIT();
			MSS_USB_device_cep_error();
        }
	}
    
#else /*MSS_USB_DEVICE_TEST_MODE*/
	usb_device_test_cb.usb_user_device_test_cep_setup(status);
#endif
}

/******************************************************************************
 This Callback function is called when Datapacket is received on Control EP.
 DATAOUT Phase.
*/
static void 
MSS_USB_device_cep_dataout_cb
(
	uint8_t status
)
{
#ifndef MSS_USB_DEVICE_TEST_MODE

	uint32_t rem_length;

	/*
    xfr_rem_length should have been setup by the Specific request which needed 
    data to arriave in the Data phase
    */
	if(status & 0x03u)
	{
		SETUP_PKT_INIT();
        device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
        g_cmd_code = 0xFFu;
        g_cep_data_buf_len = 0u;
    	MSS_USB_device_cep_read_prepare((uint8_t*)&setup_pkt , 8u);
	}
	else
	{
	
		MSS_USB_core_device_cep_read_pkt(&device_ep[MSS_USB_CTRL_EP]);
        
		if(MSS_USB_CEP_DATAOUT == device_ep[MSS_USB_CTRL_EP].state)	/*Device should be in DATAOUT phase.*/
		{
			if(device_ep[MSS_USB_CTRL_EP].xfr_count == device_ep[MSS_USB_CTRL_EP].xfr_length)
			{
				/*Call Setup request (standard or class) indicating that the desired data is arrived*/
				
				if(g_usb_class_cb->usb_class_process_requests !=0)
				{	
					device_ep->state = MSS_USB_CEP_IDLE;
					if(SUCCESS == g_usb_class_cb->usb_class_process_requests(0,
                                                                           g_cmd_code,
                                                                           0,
                                                                           (uint8_t*)&g_cep_data_buf,
                                                                           g_cep_data_buf_len))
					{
                        g_cmd_code = 0xFFu;
                        g_cep_data_buf_len = 0u;
                        SETUP_PKT_INIT();
                        device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
                        MSS_USB_device_cep_read_prepare((uint8_t*)&setup_pkt, 8u);
					}
					else
					{
                        SETUP_PKT_INIT();
                        g_cmd_code = 0xFFu;
                        g_cep_data_buf_len = 0u;
                        device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
						MSS_USB_device_cep_error();
					}
				}
			}
			else if(device_ep[MSS_USB_CTRL_EP].xfr_count < device_ep[MSS_USB_CTRL_EP].xfr_length)
			{
				/*Continue to read data on the CEP*/
				rem_length = device_ep[MSS_USB_CTRL_EP].xfr_length - device_ep[MSS_USB_CTRL_EP].xfr_count;
				
                if(rem_length >= device_ep[MSS_USB_CTRL_EP].max_pkt_size)
				{
					device_ep[MSS_USB_CTRL_EP].txn_length = device_ep[MSS_USB_CTRL_EP].max_pkt_size;
				}
				else
				{
					device_ep[MSS_USB_CTRL_EP].txn_length = rem_length;
				}
				device_ep[MSS_USB_CTRL_EP].buf_addr	+= device_ep[MSS_USB_CTRL_EP].txn_count;

				device_ep[MSS_USB_CTRL_EP].txn_count = 0u;
				MSS_USB_device_cep_read_prepare(device_ep[MSS_USB_CTRL_EP].buf_addr, device_ep[MSS_USB_CTRL_EP].txn_length);
			}
			else
			{
                SETUP_PKT_INIT();
                g_cmd_code = 0xFFu;
                g_cep_data_buf_len = 0u;
                device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
                MSS_USB_device_cep_error();
			}
		}
		else
		{
		    ASSERT(0);
		}
	}
#else /*MSS_USB_DEVICE_TEST_MODE*/
	usb_device_test_cb.usb_user_device_test_cep_dataout(status);
#endif
}

/******************************************************************************
 This Callback function is executed when Data packet is sent from control EP.
 DATAIN phase.
*/
static void 
MSS_USB_device_cep_datain_cb
(
	uint8_t status
)
{
#ifndef MSS_USB_DEVICE_TEST_MODE

	/*
    xfr_rem_length should have been setup by the Specific request which needed 
    data to arriave in the Data phase
    */
	if(status & 0x03u)
	{
		SETUP_PKT_INIT();
        device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
        g_cmd_code = 0xFFu;
        g_cep_data_buf_len = 0u;
		MSS_USB_device_cep_read_prepare((uint8_t*)&setup_pkt, 8u);
	}
	else
	{
        /*Device should be in DATAIN phase.*/
		if(MSS_USB_CEP_DATAIN == device_ep[MSS_USB_CTRL_EP].state)	
		{
			if(device_ep[MSS_USB_CTRL_EP].xfr_count < device_ep[MSS_USB_CTRL_EP].xfr_length)
			{
				/*Continue to transmit more data*/
				device_ep[MSS_USB_CTRL_EP].buf_addr += device_ep[MSS_USB_CTRL_EP].txn_count;
                
				if((device_ep[MSS_USB_CTRL_EP].xfr_length - device_ep[MSS_USB_CTRL_EP].xfr_count) >= 
                                                                device_ep[MSS_USB_CTRL_EP].max_pkt_size)
				{	
					device_ep[MSS_USB_CTRL_EP].txn_length = device_ep[MSS_USB_CTRL_EP].max_pkt_size;							
				}
				else
				{
					device_ep[MSS_USB_CTRL_EP].txn_length = 
                                        (device_ep[MSS_USB_CTRL_EP].xfr_length -
                                         device_ep[MSS_USB_CTRL_EP].xfr_count);
				}
				
                /*
                Reset the txn_count since one transaction out of the transfer is 
                completed now
                */
				device_ep[MSS_USB_CTRL_EP].txn_count = 0u;
				MSS_USB_core_device_cep_write_pkt(&device_ep[MSS_USB_CTRL_EP]);
			}
			else
			{
				device_ep[MSS_USB_CTRL_EP].txn_count = 0u;
				device_ep[MSS_USB_CTRL_EP].xfr_count = 0u;
				device_ep[MSS_USB_CTRL_EP].xfr_length = 0u;
				device_ep[MSS_USB_CTRL_EP].txn_length = 0u;
				device_ep[MSS_USB_CTRL_EP].state = MSS_USB_CEP_IDLE;
				SETUP_PKT_INIT();
				MSS_USB_device_cep_read_prepare((uint8_t*)&setup_pkt, 8u);
				/*
                Call Setup request (standard or class) indicating that 
                the desired data is transmitted
                */            
			}
		}
	}

#else /*MSS_USB_DEVICE_TEST_MODE*/
		usb_device_test_cb.usb_user_device_test_cep_datain(status);
#endif
}
/******************************************************************************
 This Callback function is executed when Data packet is received on RX EP.
 */
static void 
MSS_USB_device_ep_dataout_cb
(
	mss_usb_ep_num_t num, 
	uint8_t status
)
{
    if((status & 0x1Fu) > 1)
	{
		/*In case of any descripancies....send stall from here.*/		
        #ifndef MSS_USB_DEVICE_TEST_MODE
	        g_usb_class_cb->usb_class_dataout(num, status, device_rx_ep[num ].txn_count); 
        #else
            usb_device_test_cb.usb_user_device_test_ep_dataout(num,
                                                               status,
                                                               device_rx_ep[num].txn_count);
        #endif	
	}
	else
	{
		/*
        While receiving data, xfr_length might be known to application or may be
        unknown. When xfr_length is not known, application is expected to provide
        sizeof(Receive_buffer) as xfr_length which is big enough to receive 
        unknown data. End of data reception in Bulk transfers is flagged by 
        reception of short packet or ZLP. When DMA-m1 is enabled, the control 
        will return here only once after completion of transfer, hence we can 
        call class cb to indicate the same.
		*/
		if(device_rx_ep[num].dma_enable== DMA_ENABLE)
		{
			#ifndef MSS_USB_DEVICE_TEST_MODE
				g_usb_class_cb->usb_class_dataout(num,
                                               status,
                                               device_rx_ep[num].xfr_count); 
			#else
				usb_device_test_cb.usb_user_device_test_ep_dataout(num,
                                                                   status,
                                                                   device_rx_ep[num].xfr_count);
			#endif	
		}			

		/*
        When DMA-m1 is not enabled, control will come here for every transfer in
        the multipacket transfer.
        All the transfers will be of max_pkt_size except the last.
        Last transaction would receive short packet or ZLP.
        This indicates end of transfer.
		*/
		else 
		{
			if(device_rx_ep[num].xfr_type == MSS_USB_XFR_BULK)	
			{
				/*SHORT packet or ZLP packet was received. End of transfer.*/
				if(device_rx_ep[num].txn_count < device_rx_ep[num].max_pkt_size) 
				{
					#ifndef MSS_USB_DEVICE_TEST_MODE
						g_usb_class_cb->usb_class_dataout(num,
                                                          status,
                                                          device_rx_ep[num].txn_count); 
					#else
						usb_device_test_cb.usb_user_device_test_ep_dataout(num,
                                                                           status,
                                                                           device_rx_ep[num].txn_count);
					#endif	
				}
				else if(device_rx_ep[num].txn_count == device_rx_ep[num].max_pkt_size)
				{
					/*
                    Make sure that we expect more data to be received.
                    Andvance the buffer pointer and prepare to receive next packet.
					*/
					if(device_rx_ep[num].xfr_count < device_rx_ep[num].xfr_length)
					{
						device_rx_ep[num].buf_addr += device_rx_ep[num].txn_count;
                        
						if((device_rx_ep[num].xfr_length - device_rx_ep[num].xfr_count) >= 
                            device_rx_ep[num].max_pkt_size)
						{	
							device_rx_ep[num].txn_length = device_rx_ep[num].max_pkt_size;							
						}
						else
						{
							device_rx_ep[num].txn_length = (device_rx_ep[num].xfr_length -
                                                            device_rx_ep[num].xfr_count);
						}

						/*
                        Reset the txn_count since one transaction out of the 
                        transfer is completed now
                        */
						device_rx_ep[num].txn_count = 0u;
						MSS_USB_core_device_rx_ep_read_prepare(&device_rx_ep[num]);
					}
				}
				else
				{
					ASSERT(0);/*TODO: replace this with stall EP*/
				}
			}
            else	
			{
                /*
                For ISO and interrupt transfers, LB - expects only onr packet in 
                one transaction	HB expects at most 3 packets in one transaction.
                */
                if(device_rx_ep[num].txn_count <= device_rx_ep[num].max_pkt_size) 
				{
					#ifndef MSS_USB_DEVICE_TEST_MODE
						g_usb_class_cb->usb_class_dataout(num,
                                                          status,
                                                          device_rx_ep[num].txn_count); 
					#else
						usb_device_test_cb.usb_user_device_test_ep_dataout(num,
                                                                           status,
                                                                           device_rx_ep[num].txn_count);
					#endif	
				}
				else
				{
					ASSERT(0);/*TODO: replace this with stall EP*/
				}
			}
		}
	}
}

/******************************************************************************
 This Callback function is executed on transmission complete interrupt even when
 the core is in device mode.
*/
static void 
MSS_USB_device_ep_datain_cb
(
	mss_usb_ep_num_t num, 
	uint8_t status
)
{
	if(status > 1)
	{
	    /*There was a error in receiving packet*/
        #ifndef MSS_USB_DEVICE_TEST_MODE
	    g_usb_class_cb->usb_class_datain(num, status); 
        #else   /*MSS_USB_DEVICE_TEST_MODE*/
        usb_device_test_cb.usb_user_device_test_ep_datain(num, status);
        #endif	
	}
	else
	{
		/*
		While transmitting data we always know the xfr_length, use it to 
        check if the transfer has ended.
        
        DMA Enabled: 
        Mode-m1 is used for Bulk transfer.In this case all the data
        single or multiple packets is handled by DMA since bit is Autoset set.
        Control will return here only once when complete data is transmitted.
        Mode-m0 is used for ISO/Interrupt transfers. In this case single packet
        should have been given by the application and control will return here
        only once after the single packet is transmitted.

        DMA Disabled:
        For multi-packet bulk transfers, control will reach here after each 
        single packet is transfered. Provide next packet for tramission till the
        end of data.
        For ISO/Interrupt transfers,control comes here only once since only one
        packet is expected to be transfered.
        */
        if(DMA_ENABLE == device_ep[num].dma_enable)
        {
            if(device_ep[num].xfr_type == MSS_USB_XFR_BULK)
            {
                if(device_ep[num].xfr_count != device_ep[num].xfr_length)
                {
                    ASSERT(0);
                }
            }
            else
            {
                if(device_ep[num].txn_count != device_ep[num].txn_length)
                {
                    ASSERT(0);
                }            
            }
            #ifndef MSS_USB_DEVICE_TEST_MODE
                g_usb_class_cb->usb_class_datain(num, status); 
            #else   /*MSS_USB_DEVICE_TEST_MODE*/
                usb_device_test_cb.usb_user_device_test_ep_datain(num, status);
            #endif	
        }
        else    /*DMA_DISABLE*/
        {
            if(device_ep[num].xfr_type == MSS_USB_XFR_BULK)
            {
                if(device_ep[num].xfr_count < device_ep[num].xfr_length)
                {
                    device_ep[num].buf_addr += device_ep[num].txn_count;
                    
                    /*
                    Reset the txn_count since one transaction out of the transfer 
                    is completed now
                    */
                    device_ep[num].txn_count = 0u;
                    
                    if((device_ep[num].xfr_length - device_ep[num].xfr_count) >= 
                        device_ep[num].max_pkt_size)
                    {	
                        device_ep[num].txn_length = device_ep[num].max_pkt_size;							
                    }
                    else
                    {
                        device_ep[num].txn_length = (device_ep[num].xfr_length - 
                                                     device_ep[num].xfr_count);
                    }
                    MSS_USB_core_ep_write_pkt(&device_ep[num]);
                }
                else if(device_ep[num].xfr_count == device_ep[num].xfr_length)
                {
                    #ifndef MSS_USB_DEVICE_TEST_MODE
                    g_usb_class_cb->usb_class_datain(num, status); 
                    #else   /*MSS_USB_DEVICE_TEST_MODE*/
                    usb_device_test_cb.usb_user_device_test_ep_datain(num, status);
                    #endif	                    
                }
                else
                {
                    ASSERT(0);
                }
            }
            else    /*ISO/INTERRUPT XRF*/
            {
                if(device_ep[num].txn_count != device_ep[num].txn_length)
                {
                    ASSERT(0);
                }
                #ifndef MSS_USB_DEVICE_TEST_MODE
                g_usb_class_cb->usb_class_datain(num, status); 
                #else   /*MSS_USB_DEVICE_TEST_MODE*/
                usb_device_test_cb.usb_user_device_test_ep_datain(num, status);
                #endif	
            }
        }
	}
}

/******************************************************************************
 This Callback function is executed when suspend interrupt is received when 
 the core is in device mode.
*/
static void 
MSS_USB_device_suspend_cb
(
	void
)
{
	#ifndef MSS_USB_DEVICE_TEST_MODE
	 	MSS_USB_core_enable_irq(RESUME_IRQ_MASK);
		device_config.device_state_at_suspend = device_config.device_state;
		device_config.device_state = MSS_USB_SUSPENDED_STATE;
	#else
	/*	usb_device_test_cb->usb_user_device_test_suspend();*/
	#endif	
}

/******************************************************************************
 This Callback function is executed when resume interrupt is received when the 
 core is in device mode.
*/
static void 
MSS_USB_device_resume_cb
(
	void
)
{
	#ifndef MSS_USB_DEVICE_TEST_MODE
		device_config.device_state = device_config.device_state_at_suspend;
	#else
		usb_device_test_cb.usb_user_device_test_resume();
	#endif	
}

/******************************************************************************
 This function processes standard USB requests.
*/
static uint8_t 
MSS_USB_std_requests
(
	uint8_t** buf,
	uint32_t* length
)
{
    uint8_t result = FAIL;
    /*TODO:result should be used for all the functions*/
	switch(setup_pkt.request)
	{
	case USB_STD_REQ_GET_DESCRIPTOR:
	MSS_USB_get_descriptor(buf, length);
	break;

	case USB_STD_REQ_SET_ADDRESS: 					
	MSS_USB_set_address();
	break;

	case USB_STD_REQ_SET_CONFIG:					
	MSS_USB_set_config();
	break;

	case USB_STD_REQ_GET_CONFIG:				
	MSS_USB_get_config(buf, length);
	break;

	case USB_STD_REQ_GET_STATUS:									
	MSS_USB_get_status(buf, length);
	break;

	case USB_STD_REQ_SET_FEATURE:
    result = MSS_USB_set_feature();	 
    return result;

	case USB_STD_REQ_CLEAR_FEATURE:									
    result = 	MSS_USB_clr_feature();
    return result;

	case USB_STD_REQ_SET_DESCRIPTOR:									
	MSS_USB_set_descriptor();
	break;

   	case USB_STD_REQ_SET_INTERFACE:	
	device_config.active_interface_num = (uint8_t)setup_pkt.value;
	return SUCCESS;

 	case USB_STD_REQ_GET_INTERFACE:	
	*buf = &device_config.active_interface_num;
	*length = 1u;
	return SUCCESS;

	default:	
	return FAIL;
	}

	return SUCCESS;
}

/******************************************************************************
 This function processes USB class requests.
*/
static uint8_t 
MSS_USB_class_requests
(
	uint8_t** buf_pp,
	uint32_t* len_p
)
{
    if(setup_pkt.length)
	{
		if(setup_pkt.request_type & USB_STD_REQ_DATA_DIR_MASK)/*Device to host*/
		{
			if(g_usb_class_cb->usb_class_process_requests !=0)
			{
				if(SUCCESS == 
                    g_usb_class_cb->usb_class_process_requests(buf_pp,
                                                            setup_pkt.request,
                                                            len_p,
                                                            0,
                                                            0))
				{
					return SUCCESS;
				}
				else
				{
					return FAIL;
				}
			}
			else
			{
				return FAIL;
			}
		}
		else	/*Host to device*/
		{
			g_cmd_code = setup_pkt.request;
			g_cep_data_buf_len = setup_pkt.length; 
            
			return SUCCESS;
		}
	}
	else	/*No Data phase*/
	{
		if(g_usb_class_cb->usb_class_process_requests !=0)
		{
          	*buf_pp = (uint8_t*)&setup_pkt;
            
			if(SUCCESS ==
                g_usb_class_cb->usb_class_process_requests(0,
                                                        setup_pkt.request,
                                                        0,
                                                        (uint8_t*)&setup_pkt,
                                                        sizeof(setup_pkt)))
			{
				return SUCCESS;
			}
			else
			{
				return FAIL;
			}
		}
		else
		{
			return FAIL;
		}
	}
}

/******************************************************************************
 This function processes vendor defined USB requests.
*/
static uint8_t 
MSS_USB_vendor_requests
(
	uint8_t** buf_pp,
	uint32_t* len_p
)
{
    return FAIL;
}

static uint8_t 
MSS_USB_get_descriptor
(
	uint8_t** buf_pp,
	uint32_t* len_p
)
{
	if((device_config.device_state >= MSS_USB_DEFAULT_STATE) &&
        (USB_STD_REQ_DATA_DIR_IN == (setup_pkt.request_type & USB_STD_REQ_DATA_DIR_MASK)))
	{
        if(USB_STD_REQ_RECIPIENT_DEVICE == (setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK))
		{
			switch(setup_pkt.value >> 8)
			{
				case USB_DEVICE_DESCRIPTOR_TYPE:

					/*descriptor Index should be zero and Index field should be zero*/
					if((((uint8_t)(setup_pkt.value)) == 0) &&
                        (setup_pkt.index == 0) &&
                        (g_usb_user_descriptors_cb->usb_device_descriptor !=0))
					{
						*buf_pp = g_usb_user_descriptors_cb->usb_device_descriptor(len_p);
					}
					break;

				case USB_STRING_DESCRIPTOR_TYPE:

					/* 
					* When descriptor index is 0, index field must be 0.
					* When descriptor index is >0 , index field indicates Lang ID
					*/
					if(g_usb_user_descriptors_cb->usb_string_descriptor != 0)
					{
						*buf_pp = g_usb_user_descriptors_cb->usb_string_descriptor((uint8_t)setup_pkt.value, len_p);
					}
					break;

				case USB_DEVICE_QUALIFIER_DESCRIPTOR_TYPE:
			
					/*descriptor Index should be zero*/
					if((((uint8_t)(setup_pkt.value)) == 0) &&
                        (setup_pkt.index == 0) &&
                        (g_usb_user_descriptors_cb->usb_device_qual_descriptor)!=0)	
					{
						*buf_pp = g_usb_user_descriptors_cb->usb_device_qual_descriptor(device_config.device_speed, len_p);
					}
					break;

				case USB_CONFIGURATION_DESCRIPTOR_TYPE:
					if(g_usb_class_cb->usb_class_get_descriptor != 0)
	  				{
						*buf_pp = g_usb_class_cb->usb_class_get_descriptor(USB_STD_REQ_RECIPIENT_DEVICE,
                                                                           USB_CONFIGURATION_DESCRIPTOR_TYPE,
                                                                           len_p);
					}
					break;

				case USB_INTERFACE_DESCRIPTOR_TYPE:/*Not directly accessible, remove this*/
                    /*descriptor Index should be zero and Index field should be zero*/
					if((((uint8_t)(setup_pkt.value)) == 0u) && (setup_pkt.index == 0u))	
					{
						/*TODO: implement Interface descriptor CB*/
					}
					break;
				case USB_ENDPOINT_DESCRIPTOR_TYPE:/*Not directly accessible, remove this*/
                    /*descriptor Index should be zero and Index field should be zero*/
					if((((uint8_t)(setup_pkt.value)) == 0u) && (setup_pkt.index == 0u))
					{
						/*TODO: implement EP descriptor CB*/
					}
					break;
				case USB_OTHER_SPEED_CONFIG_DESCRIPTOR_TYPE:
                    /*descriptor Index should be zero and Index field should be zero*/
					if((((uint8_t)(setup_pkt.value)) == 0u) && (setup_pkt.index == 0u))
					{
						/*TODO: implement Other Speed descriptor CB*/
					}
					break;
				case USB_INTERFACE_POWER_DESCRIPTOR_TYPE:/*Obsolete, remove this*/
                    /*descriptor Index should be zero and Index field should be zero*/
					if((((uint8_t)(setup_pkt.value)) == 0u) && (setup_pkt.index == 0u))
					{
						/*TODO: implement Power descriptor CB*/
					}
					break;
				default:
		            MSS_USB_device_cep_error();
		            break;
			}

		}
		else if(USB_STD_REQ_RECIPIENT_INTERFACE == (setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK))
		{
			if(g_usb_class_cb->usb_class_get_descriptor != 0)
			{
				*buf_pp = g_usb_class_cb->usb_class_get_descriptor(USB_STD_REQ_RECIPIENT_INTERFACE,
                                                                   USB_HID_REPORT_DESCRIPTOR_TYPE,
                                                                   len_p);
			}
		}
		else if(USB_STD_REQ_RECIPIENT_ENDPOINT == (setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK))
		{
			if((setup_pkt.value >> 8) && (g_usb_class_cb->usb_class_get_descriptor != 0))
			{
				*buf_pp = g_usb_class_cb->usb_class_get_descriptor(USB_STD_REQ_RECIPIENT_ENDPOINT,
                                                                   (setup_pkt.value >> 8),
                                                                   len_p);
			}
		}
		if(*len_p > setup_pkt.length)
		{
			*len_p = setup_pkt.length;
		}
	}
	return SUCCESS;
}

/******************************************************************************
 This function sets the Devices address sent by host in the MSS USB register.
*/
static uint8_t
MSS_USB_set_address
(
	void
)
{
	volatile uint32_t i = 0;
    uint8_t addr;

    addr = (uint8_t)setup_pkt.value;
	device_config.device_addr = addr;

    /*
    This Delay is added to Pass compliance test GUOT-01
    After setting Dataend, Device needs to be able to respond to IN packet on
    EP0. This delay provides the window between setting dataend and setting the 
    Address register for responding to the IN packet on EP0
    
    Note: This delay may need to be increased if the Smartfusion2 is operated at 
    Higher speed. (Current testing is done at 32MHz)
    */
    for(i = 0; i < 600; i++);
	addr = (uint8_t)(setup_pkt.value & 0x7Fu);

	/*USB2.0 section 9.4.6*/
	if(device_config.device_state == MSS_USB_CONFIGURED_STATE)
	{
		/*Behavior not defined by USB2.0, May raise error here*/
		MSS_USB_device_cep_error();
		return FAIL;	
	}
	else
	{
		if(device_config.device_state == MSS_USB_DEFAULT_STATE)
		{
			if(addr != 0u)
			{
				device_config.device_state = MSS_USB_ADDRESS_STATE;			
			}
		}
		else if(device_config.device_state == MSS_USB_ADDRESS_STATE)
		{
			if(addr == 0u)
			{
				device_config.device_state = MSS_USB_DEFAULT_STATE;			
			}
		}
		MSS_USB_core_device_set_address(device_config.device_addr);
    }
	return SUCCESS;
}

/******************************************************************************
 This function processes SET_DESCRIPTOR request.
*/
static uint8_t 
MSS_USB_set_descriptor
(
	void
)
{
	return FAIL;
}

/******************************************************************************
 This function processes SET_CONFIGURATION request.
*/
static uint8_t 
MSS_USB_set_config
(
	void
)
{
	uint8_t cfgidx;

	cfgidx = (uint8_t)setup_pkt.value;

	/*USB2.0 section 9.4.6*/
	if(device_config.device_state == MSS_USB_DEFAULT_STATE)
	{
		/*Undefined.......raise error.*/
		MSS_USB_device_cep_error();
	}
	else 
	{
        /*This value will be returned in Get_config command*/
		device_config.active_config_num = cfgidx;	
		if(device_config.device_state == MSS_USB_ADDRESS_STATE)
		{
			if(cfgidx)
			{
				device_config.device_state = MSS_USB_CONFIGURED_STATE;
				g_usb_class_cb->usb_class_init(cfgidx);
			}
		}
		else if(device_config.device_state == MSS_USB_CONFIGURED_STATE)
		{
			if(cfgidx == 0)
			{
				device_config.device_state = MSS_USB_ADDRESS_STATE;
				g_usb_class_cb->usb_class_deinit(cfgidx);
			}
			else
			{
				g_usb_class_cb->usb_class_init(cfgidx);
			}
		}
	}
	return SUCCESS;
}

/******************************************************************************
 This function processes GET_CONFIGURATION requests
*/
static uint8_t 
MSS_USB_get_config
(
	uint8_t** buf_pp,
	uint32_t* len_p
)
{
	if((setup_pkt.value == 0u) && (setup_pkt.index == 0u) && (setup_pkt.length == 1u))	
	{
        /*This value was set in Set_config command*/
		*buf_pp = &device_config.active_config_num ;	
        
		*len_p = 1u;
	}
	else
	{
		return FAIL;	
	}
	return SUCCESS;
}

/******************************************************************************
 This function processes GET_STATUS request.
*/
static uint8_t 
MSS_USB_get_status 
(
	uint8_t** buf_pp,
	uint32_t* len_p
)
{
	if(device_config.device_state == MSS_USB_ADDRESS_STATE)
	{
		if((setup_pkt.value == 0u) && (setup_pkt.index == 0u) && (setup_pkt.length == 2u))		
		{
			switch (setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK)
			{
			case USB_STD_REQ_RECIPIENT_DEVICE:              /*SF2 device is always self powered*/
					g_device_status = 0x0001;				
					g_device_status |= (device_config.remote_wakeup<<0x0001u);
					*buf_pp = (uint8_t*)&g_device_status;
					break;
			case USB_STD_REQ_RECIPIENT_INTERFACE:	        /*Reserved by USB2.0*/
					g_device_status = 0x0000u;				
					*buf_pp = (uint8_t*)&g_device_status;
				break;

			case USB_STD_REQ_RECIPIENT_ENDPOINT:            /*Endpoint halt (stall) status*/
					g_device_status = ((device_ep[setup_pkt.index].stall) ? 0x0001:0x0000);		
					*buf_pp = (uint8_t*)&g_device_status;
				break;
			default:
				return FAIL;
			}
			* len_p = sizeof(g_device_status);
		}
		else
		{
			return FAIL;
		}			
	}
	else if(device_config.device_state == MSS_USB_CONFIGURED_STATE)
	{
		if((setup_pkt.value == 0) && (setup_pkt.length == 2))		
		{
			switch (setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK)
			{
			case USB_STD_REQ_RECIPIENT_DEVICE: /*SF2 device is always self powered*/
					g_device_status = 0x0001;				
					g_device_status |= (device_config.remote_wakeup << 0x0001u);
					*buf_pp = (uint8_t*)&g_device_status;
					break;
			case USB_STD_REQ_RECIPIENT_INTERFACE:
					if(setup_pkt.index <= device_config.device_total_interfaces)
					{
                        g_device_status = 0x0000u;		/*Reserved by USB2.0*/
                        *buf_pp = (uint8_t*)&g_device_status;
					}
					else
					{
						return FAIL;
					}							
					break;
			case USB_STD_REQ_RECIPIENT_ENDPOINT:
				if(setup_pkt.index <= device_config.device_total_ep)
				{
                    /*Endpoint status here*/
					g_device_status = ((device_ep[setup_pkt.index].stall) ? 0x0001:0x0000) ;		
					*buf_pp = (uint8_t*)&g_device_status;
				}
				else
				{
					return FAIL;
				}							
				break;
			default:
				return FAIL;
			}
			* len_p = sizeof(g_device_status);
		}	
        else
		{
			return FAIL;
		}			

	}
	else if(device_config.device_state == MSS_USB_DEFAULT_STATE)
	{
		return FAIL;
	}
	return SUCCESS;
}

/******************************************************************************
 This function processes SET_FEATURE request.
*/
static uint8_t 
MSS_USB_set_feature
(
	void
)
{
	uint8_t result = SUCCESS;
	if(setup_pkt.length == 0)
	{
		switch(setup_pkt.value)
		{
		case USB_STD_FEATURE_REMOTE_WAKEUP:
			if(((setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK) == USB_STD_REQ_RECIPIENT_DEVICE) &&
                (device_config.device_state > MSS_USB_DEFAULT_STATE))
			{
                /*Enable Remote wakeup capability for the device*/
				device_config.remote_wakeup = 0x01u;		
			}
			else
			{
				result = FAIL;
			}
			break;
		case USB_STD_FEATURE_EP_HALT:
			if(((setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK) == USB_STD_REQ_RECIPIENT_ENDPOINT) &&
                ((uint8_t)(setup_pkt.index)) &&
                (device_config.device_state == MSS_USB_CONFIGURED_STATE))
			{
				/*8.5.3.4 Control EP should not be implementing HALT feature*/
				if(((uint8_t)(setup_pkt.index) & 0x80u)) /* IN,TX endpoint*/
				{
					device_ep[(((uint8_t)(setup_pkt.index))& 0x7fu)].stall = 0x01; 	/*Enable HALT*/
					MSS_USB_core_tx_ep_stall((mss_usb_ep_num_t)((((uint8_t)(setup_pkt.index)) & 0x7fu)));
				}
				else	/*out, rx endpoint*/
				{
					device_rx_ep[(uint8_t)(setup_pkt.index)].stall = 0x01; 	/*Enable HALT*/
					MSS_USB_core_rx_ep_stall((mss_usb_ep_num_t)(setup_pkt.index));
				}
			}
			else
			{
				result = FAIL;
			}
			break;
		case USB_STD_FEATURE_TEST_MODE:
			if((setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK) == USB_STD_REQ_RECIPIENT_DEVICE)
			{
				switch(setup_pkt.index >> 8u)
				{
					case USB_TEST_MODE_SELECTOR_TEST_J:
					break;
					case USB_TEST_MODE_SELECTOR_TEST_K:
					break;
					case USB_TEST_MODE_SELECTOR_TEST_SE0NAK:
					break;
					case USB_TEST_MODE_SELECTOR_TEST_PACKET:
					break;
					case USB_TEST_MODE_SELECTOR_TEST_FORCE_ENA:
					break;
					default:
					break;
				}
			}
			else
			{
				result = FAIL;
			}
			break;
		default:
			result = FAIL;
			break;				
		}
	}
	else 
	{
		result = FAIL;
	}
	return result;
}

/******************************************************************************
 This function processes CLEAR_FEATURE.
*/
static uint8_t 
MSS_USB_clr_feature
(
	void
)
{
	uint8_t result = SUCCESS;
    
	if((setup_pkt.length == 0) && (device_config.device_state > MSS_USB_DEFAULT_STATE))
	{
		switch(setup_pkt.value)
		{
		case USB_STD_FEATURE_REMOTE_WAKEUP:
			if((setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK) == USB_STD_REQ_RECIPIENT_DEVICE)
			{
                /*Disable Remote wakeup capability for the device*/
				device_config.remote_wakeup = 0x00;		
            }
			break;
		case USB_STD_FEATURE_EP_HALT:
			if(((setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK) == USB_STD_REQ_RECIPIENT_ENDPOINT))
			{
                /*8.5.3.4 HALT clear can be performed on CEP and Data EP*/
				if((uint8_t)(setup_pkt.index)) /*Not a control endpoint*/
				{
					if(((uint8_t)(setup_pkt.index) & 0x80)) /* IN,TX endpoint*/
					{
                        /*Disable HALT*/
						device_ep[(((uint8_t)(setup_pkt.index))& 0x7f)].stall = 0x00; 	
						MSS_USB_core_tx_ep_clr_stall((mss_usb_ep_num_t)((((uint8_t)(setup_pkt.index)) & 0x7f)));
					}
					else	/* out, rx endpoint */
					{
                        /*Enable HALT*/
						device_rx_ep[(uint8_t)(setup_pkt.index)].stall = 0x00; 	
						MSS_USB_core_rx_ep_clr_stall((mss_usb_ep_num_t)(setup_pkt.index));
					}
				}
				else
				{
					result = SUCCESS;
				}
			}
			break;
		case USB_STD_FEATURE_TEST_MODE:
			if((setup_pkt.request_type & USB_STD_REQ_RECIPIENT_MASK) == USB_STD_REQ_RECIPIENT_DEVICE)
			{
				switch(setup_pkt.index >> 8)
				{
					case USB_TEST_MODE_SELECTOR_TEST_J:
					break;
					case USB_TEST_MODE_SELECTOR_TEST_K:
					break;
					case USB_TEST_MODE_SELECTOR_TEST_SE0NAK:
					break;
					case USB_TEST_MODE_SELECTOR_TEST_PACKET:
					break;
					case USB_TEST_MODE_SELECTOR_TEST_FORCE_ENA:
					break;
					default:
					break;
				}
			}
			break;
		default:
			result = FAIL;
			break;				
		}
	}
	else 
	{
		result = FAIL;
	}
	return result;
}

#ifdef __cplusplus
}
#endif
