/*******************************************************************************
 * (c) Copyright 2011 Actel Corporation.  All rights reserved.
 * 
 * Public function declarations, register and bit leve definitions for 
 * G4M ethernet MAC SGMII interface.
 * This file contains bit definitions of only SGMII specific registers. 
 * For bit definitions of other (standard MII) registers, refer phy.h.
 * 
 * 
 * SVN $Revision: 3526 $
 * SVN $Date: 2011-04-22 14:53:25 +0530 (Fri, 22 Apr 2011) $
 */
#ifndef MSS_ETHERNET_SGMII_PHY_H_
#define MSS_ETHERNET_SGMII_PHY_H_

#ifdef __cplusplus
extern "C" {
#endif

/*------------------------------------------------------------------------------
 * SGMII PHY address
 */
#define SGMII_PHY_ADDR                          0x1E

/*------------------------------------------------------------------------------
 * SGMII PHY specific register offsets
 */
#define SGMII_AN_ADVT                           0x04
#define SGMII_AN_BPABLTY                        0x05
#define SGMII_AN_NXTPGTX                        0x07
#define SGMII_JITTER_DIAG                       0x10
#define SGMII_TBI_CONTROL                       0x11

/*------------------------------------------------------------------------------
 * SGMII Auto negotiation advertisement control register
 */
#define AN_ADVT_RESERVED                       0x3FFu
#define AN_ADVT_RESERVED_DEF_VAL               0x001u
#define AN_ADVT_FULL_DUPLEX                    (1u<<12)
#define AN_ADVT_ACK_RO                         (1u<<14)
#define AN_ADVT_LINKUP                         (1u<<15)

#define AN_ADVT_10MBPS                         (0u<<10)
#define AN_ADVT_100MBPS                        (1u<<10)
#define AN_ADVT_1000MBPS                       (2u<<10)
#define AN_ADVT_RSVD_LINK_SPEED                (3u<<10)

/*------------------------------------------------------------------------------
 * SGMII Auto negotiation link partner's base page ability register
 */                                     
#define AN_BPABLTY_LINK_SPEED                  (3u<<10)
#define AN_BPABLTY_FULL_DUPLEX                 (1u<<12)
#define AN_BPABLTY_ACK_RO                      (1u<<14)
#define AN_BPABLTY_LINKUP                      (1u<<15)

/*------------------------------------------------------------------------------
 * SGMII Auto negotiation next page transmit register definitions
 */
#define AN_NXTPGTX_UNFCODE                      (0x3FFu)
#define AN_NXTPGTX_TOGGLE                       (1u<<11)
#define AN_NXTPGTX_ACK2                         (1u<<12)
#define AN_NXTPGTX_MSGPAGE                      (1u<<13)
#define AN_NXTPGTX_NEXTPAGE                     (1u<<15)

/*------------------------------------------------------------------------------
 * Jitter diagnostics register definitions
 */
#define JITTER_CUST_PATTERN                     0x03FFu
#define JITTER_CUSTOM_SELECT                    (0u<<12)
#define JITTER_36A_HIGHF_SELECT                 (1u<<12)
#define JITTER_36A_MIXDF_SELECT                 (2u<<12)
#define JITTER_CUST_LOWF_SELECT                 (3u<<12)
#define JITTER_RANDOM_SELECT                    (4u<<12)
#define JITTER_36A_LOWF_SELECT                  (5u<<12)

/*------------------------------------------------------------------------------
 * TBI Control register definitions
 */
#define TBI_COMMA_DETECT                        (1u<<0)
#define TBI_ENABLE_WRAP                         (1u<<1)
#define TBI_GMII_MODE                           (1u<<4)
#define TBI_RX_CLK_SELECT                       (1u<<5)
#define TBI_AUTO_NEG_SENSE                      (1u<<8)
#define TBI_TX_RUN_DISPARITY_DISABLE            (1u<<12)
#define TBI_RX_RUN_DISPARITY_DISABLE            (1u<<13)
#define TBI_SHORTCUT_LINK_TIMER                 (1u<<14)
#define TBI_SOFT_RESET                          (1u<<15)


/**************************************************************************/
/* Public Function declarations                                           */
/**************************************************************************/

/***************************************************************************//**
  The function sgmii_init() is used to initialize internal SGMII PHY interface
  of G4M Ethernet MAC. This function is called from MSS_MAC_init() function when
  SGMII is the selected PHY interface. PHY interface callback mechanism defined
  in PHY_devices of phy.c is used to make this funciton call. This function 
  resets the SGMII PHY interface, disables loopback, enables auto-negotiation 
  and restarts the same when auto-negotiation is enabled in the MAC 
  configuration record. 
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer.
 
  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"

    PHY_device[cfg->phy].init( cfg );
  @endcode
 */
return_t 
sgmii_init
(
    MAC_cfg_t * cfg
);

/***************************************************************************//**
  The function sgmii_config_aneg() is used to configure and perform auto
  negotiation for SGMII PHY interface. This function is called from MSS_MAC_init()
  using PHY_device callback mechanism when the PHY interface is configured as SGMII
  by the user. This functions advertises the capabilities configured by the user
  after complying with the SGMII PHY capabilities. Then it restarts auto
  negotiation and waits till it is completed. This function internally polls the 
  "auto negotiation complete" flag of SGMII PHY status register and returns once
  the flag indicates that auto negotiation is completed or timed out.
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer.

  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"

    PHY_device[cfg->phy].config_aneg( cfg );
  @endcode
 */
return_t 
sgmii_config_aneg
(
    MAC_cfg_t * cfg
);

/***************************************************************************//**
  The function sgmii_read_status() is used to read the status of the SGMII PHY
  once initiated auto-negoation is completed. This function uses the cfg
  parameter as out param. This function reads the link partnet ability register
  of SGMII to find the negotiated abilities. Accordingly some of the elements
  of configuration record will be modified, which will be used in initialization 
  of Ethernet MAC. 
 
  @param cfg
    The cfg parameter is the MAC configuration record provided by the user. This
    is of MAC_cfg_t type pointer. Some elements of this record will be modified
    after successful execution of this function. 

  @return
    This function returns 1 on successful execution, returns 0 otherwise. 

  Example:
  @code
    #include "mss_ethernet_mac.h"
    #include "phy.h"

    PHY_device[cfg->phy].read_status( cfg );
  @endcode
 */
return_t 
sgmii_read_status
(
    MAC_cfg_t * cfg
);

#ifdef __cplusplus
}
#endif

#endif /* MSS_ETHERNET_SGMII_PHY_H_ */


